# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
Access to physical memory

usage:
    >>> read_physical_mem( 0xf0000, 0x100 )
    >>> write_physical_mem( 0xf0000, 0x100, buffer )
    >>> write_physical_mem_dowrd( 0xf0000, 0xdeadbeef )
    >>> read_physical_mem_dowrd( 0xfed40000 )
"""

from struct import unpack, pack
from typing import Tuple, Optional
from chipsec.hal.hal_base import HALBase
from chipsec.library.logger import print_buffer_bytes


class Memory(HALBase):
    def __init__(self, cs):
        super(Memory, self).__init__(cs)
        self.helper = cs.helper

    ####################################################################################
    #
    # Physical memory API using 64b Physical Address
    # (Same functions as below just using 64b PA instead of High and Low 32b parts of PA)
    #
    ####################################################################################

    # Reading physical memory

    def read_physical_mem(self, phys_address: int, length: int) -> bytes:
        self.logger.log_hal(f'[mem] 0x{phys_address:016X}')
        return self.helper.read_phys_mem(phys_address, length)

    def read_physical_mem_qword(self, phys_address: int) -> int:
        out_buf = self.read_physical_mem(phys_address, 8)
        value = unpack('=Q', out_buf)[0]
        self.logger.log_hal(f'[mem] qword at PA = 0x{phys_address:016X}: 0x{value:016X}')
        return value

    def read_physical_mem_dword(self, phys_address: int) -> int:
        out_buf = self.read_physical_mem(phys_address, 4)
        value = unpack('=I', out_buf)[0]
        self.logger.log_hal(f'[mem] dword at PA = 0x{phys_address:016X}: 0x{value:08X}')
        return value

    def read_physical_mem_word(self, phys_address: int) -> int:
        out_buf = self.read_physical_mem(phys_address, 2)
        value = unpack('=H', out_buf)[0]
        self.logger.log_hal(f'[mem] word at PA = 0x{phys_address:016X}: 0x{value:04X}')
        return value

    def read_physical_mem_byte(self, phys_address: int) -> int:
        out_buf = self.read_physical_mem(phys_address, 1)
        value = unpack('=B', out_buf)[0]
        self.logger.log_hal(f'[mem] byte at PA = 0x{phys_address:016X}: 0x{value:02X}')
        return value

    # Writing physical memory

    def write_physical_mem(self, phys_address: int, length: int, buf: bytes) -> int:
        if self.logger.HAL:
            self.logger.log(f'[mem] buffer len = 0x{length:X} to PA = 0x{phys_address:016X}')
            print_buffer_bytes(buf)
        return self.helper.write_phys_mem(phys_address, length, buf)

    def write_physical_mem_dword(self, phys_address: int, dword_value: int) -> int:
        self.logger.log_hal(f'[mem] dword to PA = 0x{phys_address:016X} <- 0x{dword_value:08X}')
        return self.write_physical_mem(phys_address, 4, pack('I', dword_value))

    def write_physical_mem_word(self, phys_address: int, word_value: int) -> int:
        self.logger.log_hal(f'[mem] word to PA = 0x{phys_address:016X} <- 0x{word_value:04X}')
        return self.write_physical_mem(phys_address, 2, pack('H', word_value))

    def write_physical_mem_byte(self, phys_address: int, byte_value: int) -> int:
        self.logger.log_hal(f'[mem] byte to PA = 0x{phys_address:016X} <- 0x{byte_value:02X}')
        return self.write_physical_mem(phys_address, 1, pack('B', byte_value))

    # Allocate physical memory buffer

    def alloc_physical_mem(self, length: int, max_phys_address: int = 0xFFFFFFFFFFFFFFFF) -> Tuple[int, int]:
        (va, pa) = self.helper.alloc_phys_mem(length, max_phys_address)
        self.logger.log_hal(f'[mem] Allocated: PA = 0x{pa:016X}, VA = 0x{va:016X}')
        return (va, pa)

    def va2pa(self, va: int) -> Optional[int]:
        (pa, error_code) = self.helper.va2pa(va)
        if error_code:
            self.logger.log_hal(f'[mem] Looks like VA (0x{va:016X}) not mapped')
            return None
        self.logger.log_hal(f'[mem] VA (0x{va:016X}) -> PA (0x{pa:016X})')
        return pa

    # Map physical address to virtual

    def map_io_space(self, pa: int, length: int, cache_type: int) -> int:
        va = self.helper.map_io_space(pa, length, cache_type)
        self.logger.log_hal(f'[mem] Mapped: PA = 0x{pa:016X}, VA = 0x{va:016X}')
        return va

    # Free physical memory buffer

    def free_physical_mem(self, pa: int) -> bool:
        ret = self.helper.free_phys_mem(pa)
        self.logger.log_hal(f'[mem] Deallocated : PA = 0x{pa:016X}')
        return True if ret == 1 else False

    def set_mem_bit(self, addr: int, bit: int) -> int:
        addr += bit >> 3
        byte = self.read_physical_mem_byte(addr)
        self.write_physical_mem_byte(addr, (byte | (0x1 << (bit & 0x7))))
        return byte

    # reference: https://github.com/tianocore/edk2/blob/c4fdec0a83d69bd0399b1b4351fa9c3af3c6fd65/UefiCpuPkg/Library/MtrrLib/MtrrLib.c#L816
    def get_max_memory_bit_size(self):
        CPU_EXTENDED_FUNCTION = 0x80000000
        CPU_VIR_PHY_ADDRESS_SIZE = 0x80000008
        CPUID_SIG = 0
        CPUID_STRUCTURED_EXTENDED_FEATURE_FLAGS = 0x7
        NONINTEL_DEFAULT_MAX_PA_SIZE = 36
        MSR_IA32_TME_ACTIVATE = 0x982
        MSR_TME_ACTIVATE_ENABLED_BIT = 1
        MSR_TME_ACTIVATE_ENABLED_BIT_SIZE = 1
        MSR_TME_ACTIVATE_MKTME_KEYID_BIT = 32
        MSR_TME_ACTIVATE_MKTME_KEYID_BIT_SIZE = 4
        phys_address_size = 0
        DEFAULT_THREAD_ID = 0
        eax, _, _, _ = self.helper.cpuid(CPU_EXTENDED_FUNCTION, 0)
        if eax >= CPU_VIR_PHY_ADDRESS_SIZE:
            eax, _, _, _ = self.helper.cpuid(CPU_VIR_PHY_ADDRESS_SIZE, 0)
            phys_address_size = self.get_value_from_bit_def(eax, 0, 8)
        else:
            phys_address_size = NONINTEL_DEFAULT_MAX_PA_SIZE  # If not defined in CPUID, assume size 36 and non-standard Intel CPU
        max_func, _, _, _ = self.helper.cpuid(CPUID_SIG, 0)
        if max_func >= 0x7:
            _, _, ex_feature, _ = self.helper.cpuid(CPUID_STRUCTURED_EXTENDED_FEATURE_FLAGS, 0)
            ex_func = self.get_value_from_bit_def(ex_feature, 13, 1)
            if ex_func == 1:
                tme_active = self.helper.read_msr(DEFAULT_THREAD_ID, MSR_IA32_TME_ACTIVATE)
                if self.get_value_from_bit_def(tme_active, MSR_TME_ACTIVATE_ENABLED_BIT, MSR_TME_ACTIVATE_ENABLED_BIT_SIZE) > 0:
                    phys_address_size -= self.get_value_from_bit_def(tme_active, MSR_TME_ACTIVATE_MKTME_KEYID_BIT, MSR_TME_ACTIVATE_MKTME_KEYID_BIT_SIZE)
        return phys_address_size

    def get_max_memory_mask(self):
        size = self.get_max_memory_bit_size()
        return self.get_bit_mask(size)

    def get_bit_mask(self, size):
        return (1 << size) - 1

    def get_value_from_bit_def(self, inputValue, fieldStartBit, fieldSize):
        return (inputValue >> fieldStartBit) & self.get_bit_mask(fieldSize)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2016, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Oracle VirtualBox CVE-2015-0377 check

Reference:
    - PoC test for Host OS Crash when writing to IA32_APIC_BASE MSR (Oracle VirtualBox CVE-2015-0377)
    - http://www.oracle.com/technetwork/topics/security/cpujan2015-1972971.html

Usage:
   ``chipsec_main.py -i -m tools.vmm.vbox_crash_apicbase``

Examples:
    >>> chipsec_main.py -i -m tools.vmm.vbox_crash_apicbase

Registers used:
    - IA32_APIC_BASE

.. warning::
    - Module can cause VMM/Host OS to crash; if so, this is a FAILURE

"""

from chipsec.library.exceptions import HWAccessViolationError
from chipsec.module_common import BaseModule, CPU
from chipsec.library.returncode import ModuleResult

_MODULE_NAME = 'vbox_crash_apicbase'

TAGS = [CPU]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'VMM', 'VBOX', 'VBOX_CRASH_APICBASE']


class vbox_crash_apicbase(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)

    def run(self, module_argv):
        self.logger.start_test('Host OS Crash due to IA32_APIC_BASE (Oracle VirtualBox CVE-2015-0377)')

        tid = 0
        apicbase_msr = self.cs.register.read('IA32_APIC_BASE', tid)
        self.cs.register.print('IA32_APIC_BASE', apicbase_msr)
        apicbase_msr = 0xDEADBEEF00000000 | (apicbase_msr & 0xFFFFFFFF)
        self.logger.log(f'[*] Writing 0x{apicbase_msr:016X} to IA32_APIC_BASE MSR..')
        try:
            self.cs.register.write('IA32_APIC_BASE', apicbase_msr, tid)
        except HWAccessViolationError:
            self.logger.log('System blocked write attempt.')

        # If we are here, then we are fine ;)
        self.logger.log_passed("VMM/Host OS didn't crash (not vulnerable)")
        self.result.setStatusBit(self.result.status.SUCCESS)
        self.res = self.result.getReturnCode(ModuleResult.PASSED)
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2018, Eclypsium, Inc.
# Copyright (c) 2018-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

"""
This module checks if the system has debug features turned on,
specifically the Direct Connect Interface (DCI).

This module checks the following bits:
1. HDCIEN bit in the DCI Control Register
2. Debug enable bit in the IA32_DEBUG_INTERFACE MSR
3. Debug lock bit in the IA32_DEBUG_INTERFACE MSR
4. Debug occurred bit in the IA32_DEBUG_INTERFACE MSR

Usage:
    ``chipsec_main -m common.debugenabled``

Examples:
    >>> chipsec_main.py -m common.debugenabled

The module returns the following results:
    - **FAILED** : Any one of the debug features is enabled or unlocked.
    - **PASSED** : All debug feature are disabled and locked.

Registers used:
    - IA32_DEBUG_INTERFACE[DEBUGENABLE]
    - IA32_DEBUG_INTERFACE[DEBUGELOCK]
    - IA32_DEBUG_INTERFACE[DEBUGEOCCURED]
    - P2SB_DCI.DCI_CONTROL_REG[HDCIEN]

"""

from chipsec.module_common import BaseModule, CPU
from chipsec.library.returncode import ModuleResult
from chipsec.library.defines import BIT11
from typing import List

TAGS = [CPU]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'DEBUGENABLED']

_MODULE_NAME = 'debugenabled'


class debugenabled(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)
        self.is_enable_set = False
        self.is_debug_set = False
        self.is_lock_set = True

    def is_supported(self) -> bool:
        # Use CPUID Function 1 to determine if the IA32_DEBUG_INTERFACE MSR is supported.
        # See IA32 SDM CPUID Instruction for details.  (SDBG ECX bit 11)
        (_, _, ecx, _) = self.cs.cpu.cpuid(1, 0)
        supported = (ecx & BIT11) != 0
        if not supported and not self.cs.register.is_defined('ECTRL'):
            self.logger.log_important('CPU Debug features are not supported on this platform.  Skipping module.')
        return supported

    def check_dci(self) -> int:
        TestFail = ModuleResult.PASSED
        self.logger.log('')
        self.logger.log('[*] Checking DCI register status')
        ectrl = self.cs.register.read('ECTRL')
        HDCIEN = self.cs.register.get_field('ECTRL', ectrl, 'ENABLE') == 1
        if self.logger.VERBOSE:
            self.cs.register.print('ECTRL', ectrl)
        if HDCIEN:
            self.logger.log_bad('DCI Debug is enabled')
            TestFail = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.DEBUG_FEATURE)
        else:
            self.logger.log_good('DCI Debug is disabled')
        return TestFail

    def check_cpu_debug_enable(self) -> int:
        self.logger.log('')
        self.logger.log('[*] Checking IA32_DEBUG_INTERFACE MSR status')
        TestFail = ModuleResult.PASSED
        for tid in range(self.cs.msr.get_cpu_thread_count()):
            dbgiface = self.cs.register.read('IA32_DEBUG_INTERFACE', tid)
            IA32_DEBUG_INTERFACE_DEBUGENABLE = self.cs.register.get_field('IA32_DEBUG_INTERFACE', dbgiface, 'ENABLE') == 1
            IA32_DEBUG_INTERFACE_DEBUGELOCK = self.cs.register.get_field('IA32_DEBUG_INTERFACE', dbgiface, 'LOCK') == 1
            IA32_DEBUG_INTERFACE_DEBUGEOCCURED = self.cs.register.get_field('IA32_DEBUG_INTERFACE', dbgiface, 'DEBUG_OCCURRED') == 1

            if self.logger.VERBOSE:
                self.cs.register.print('IA32_DEBUG_INTERFACE', dbgiface)

            if IA32_DEBUG_INTERFACE_DEBUGENABLE:
                self.logger.log_bad('CPU debug enable requested by software.')
                self.is_enable_set = True
                TestFail = ModuleResult.FAILED
                self.result.setStatusBit(self.result.status.DEBUG_FEATURE)
            if not IA32_DEBUG_INTERFACE_DEBUGELOCK:
                self.logger.log_bad('CPU debug interface is not locked.')
                self.is_lock_set = False
                TestFail = ModuleResult.FAILED
                self.result.setStatusBit(self.result.status.LOCKS)
            if IA32_DEBUG_INTERFACE_DEBUGEOCCURED:
                self.logger.log_important('Debug Occurred bit set in IA32_DEBUG_INTERFACE MSR')
                self.is_debug_set = True
                self.result.setStatusBit(self.result.status.DEBUG_FEATURE)
                if TestFail == ModuleResult.PASSED:
                    TestFail = ModuleResult.WARNING
            if TestFail == ModuleResult.PASSED:
                self.logger.log_good('CPU debug interface state is correct.')
        return TestFail

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('Debug features test')

        cpu_debug_test_fail = self.check_cpu_debug_enable()

        dci_test_fail = ModuleResult.PASSED
        if self.cs.register.is_defined('ECTRL'):
            dci_test_fail = self.check_dci()

        self.logger.log('')
        self.logger.log('[*] Module Results:')

        if self.is_debug_set:
            self.logger.log_important('IA32_DEBUG_INTERFACE.DEBUG_OCCURRED bit is set.')
        if self.is_enable_set:
            self.logger.log_important('IA32_DEBUG_INTERFACE.ENABLE bit is set.')
        if not self.is_lock_set:
            self.logger.log_important('IA32_DEBUG_INTERFACE.LOCK bit is NOT set.')

        if self.cs.control.is_defined('SamplePart'):
            if self.cs.control.get('SamplePart') == 1:
                self.logger.log_passed('CPU is a Sample Part. Test is N/A.')
                self.res = self.result.getReturnCode(ModuleResult.INFORMATION)
                return self.res

        if (dci_test_fail == ModuleResult.FAILED) or (cpu_debug_test_fail == ModuleResult.FAILED):
            self.logger.log_failed('One or more of the debug checks have failed and a debug feature is enabled')
            self.res = self.result.getReturnCode(ModuleResult.FAILED)
        elif (dci_test_fail == ModuleResult.WARNING) or (cpu_debug_test_fail == ModuleResult.WARNING):
            self.logger.log_warning('An unexpected debug state was discovered on this platform')
            self.res = self.result.getReturnCode(ModuleResult.WARNING)
        else:
            self.logger.log_passed('All checks have successfully passed')
            self.res = self.result.getReturnCode(ModuleResult.PASSED)

        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
>>> chipsec_util ucode id|load|decode [ucode_update_file (in .PDB or .BIN format)] [cpu_id]

Examples:

>>> chipsec_util ucode id
>>> chipsec_util ucode load ucode.bin 0
>>> chipsec_util ucode decode ucode.pdb
"""

from chipsec.command import BaseCommand, toLoad
from chipsec.library.file import read_file
from chipsec.hal.ucode import dump_ucode_update_header
from argparse import ArgumentParser

# ###################################################################
#
# Microcode patches
#
# ###################################################################


class UCodeCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.Driver

    def parse_arguments(self) -> None:
        parser = ArgumentParser(usage=__doc__)
        subparsers = parser.add_subparsers()
        parser_id = subparsers.add_parser('id')
        parser_id.add_argument('cpu_thread_id', nargs='?', type=lambda x: int(x, 16), default=None, help='CPU ID (hex)')
        parser_id.set_defaults(func=self.ucode_id)

        parser_load = subparsers.add_parser('load')
        parser_load.add_argument('ucode_filename', type=str, help='ucode file name (.PDB or .BIN format)')
        parser_load.add_argument('cpu_thread_id', nargs='?', type=lambda x: int(x, 16), default=None, help='CPU ID (hex)')
        parser_load.set_defaults(func=self.ucode_load)

        parser_decode = subparsers.add_parser('decode')
        parser_decode.add_argument('ucode_filename', type=str, help='ucode file name (.PDB format)')
        parser.parse_args(self.argv, namespace=self)

    def ucode_id(self):
        if self.cpu_thread_id is None:
            for tid in range(self.cs.msr.get_cpu_thread_count()):
                ucode_update_id = self.cs.ucode.ucode_update_id(tid)
                self.logger.log("[CHIPSEC] CPU{:d}: Microcode update ID = 0x{:08X}".format(tid, ucode_update_id))
        else:
            ucode_update_id = self.cs.ucode.ucode_update_id(self.cpu_thread_id)
            self.logger.log("[CHIPSEC] CPU{:d}: Microcode update ID = 0x{:08X}".format(self.cpu_thread_id, ucode_update_id))

    def ucode_load(self):
        if self.cpu_thread_id is None:
            self.logger.log("[CHIPSEC] Loading Microcode update on all cores from '{}'".format(self.ucode_filename))
            self.cs.ucode.update_ucode_all_cpus(self.ucode_filename)
        else:
            self.logger.log("[CHIPSEC] Loading Microcode update on CPU{:d} from '{}'".format(self.cpu_thread_id, self.ucode_filename))
            self.cs.ucode.update_ucode(self.cpu_thread_id, self.ucode_filename)

    def ucode_decode(self):
        if (not self.ucode_filename.endswith('.pdb')):
            self.logger.log("[CHIPSEC] Ucode update file is not PDB file: '{}'".format(self.ucode_filename))
            return
        pdb_ucode_buffer = read_file(self.ucode_filename)
        self.logger.log("[CHIPSEC] Decoding Microcode Update header of PDB file: '{}'".format(self.ucode_filename))
        dump_ucode_update_header(pdb_ucode_buffer)


commands = {'ucode': UCodeCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2023, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

""""
To execute: python[3] -m unittest tests.utilcmd.cmos_cmd.test_cmos_cmd
"""

import unittest
import os

from chipsec.library.file import get_main_dir
from chipsec.testcase import ExitCode
from tests.utilcmd.run_chipsec_util import setup_run_destroy_util


class TestCmosUtilcmd(unittest.TestCase):
    def test_dump(self) -> None:
        init_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "adlenumerate.json")
        cmos_dump_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "cmos_cmd", "cmos_cmd_dump_1.json")
        retval = setup_run_destroy_util(init_replay_file, "cmos", "dump", util_replay_file=cmos_dump_replay_file)
        self.assertEqual(retval, ExitCode.OK)

    def test_readl(self) -> None:
        init_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "adlenumerate.json")
        cmos_readl_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "cmos_cmd", "cmos_cmd_readl_1.json")
        retval = setup_run_destroy_util(init_replay_file, "cmos", "readl 0x0", util_replay_file=cmos_readl_replay_file)
        self.assertEqual(retval, ExitCode.OK)

    def test_writel(self) -> None:
        init_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "adlenumerate.json")
        cmos_writel_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "cmos_cmd", "cmos_cmd_writel_1.json")
        retval = setup_run_destroy_util(init_replay_file, "cmos", "writel 0x0 0xCC", util_replay_file=cmos_writel_replay_file)
        self.assertEqual(retval, ExitCode.OK)

    def test_readh(self) -> None:
        init_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "adlenumerate.json")
        cmos_readh_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "cmos_cmd", "cmos_cmd_readh_1.json")
        retval = setup_run_destroy_util(init_replay_file, "cmos", "readh 0x0", util_replay_file=cmos_readh_replay_file)
        self.assertEqual(retval, ExitCode.OK)

    def test_writeh(self) -> None:
        init_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "adlenumerate.json")
        cmos_writeh_replay_file = os.path.join(get_main_dir(), "tests", "utilcmd", "cmos_cmd", "cmos_cmd_writeh_1.json")
        retval = setup_run_destroy_util(init_replay_file, "cmos", "writeh 0x0 0xCC", util_replay_file=cmos_writeh_replay_file)
        self.assertEqual(retval, ExitCode.OK)


if __name__ == '__main__':
    unittest.main()




# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2023, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

import unittest
from unittest.mock import patch, Mock, MagicMock, call
from chipsec.hal.cpu import CPU


class TestHalCpu(unittest.TestCase):

    def test_hal_cpu_init(self):
        mock_cs = MagicMock()
        dci = CPU(mock_cs)
        self.assertIsInstance(dci, CPU)

    def test_hal_cpu_read_cr(self):
        mock_self = Mock()
        test_value = 0x8086
        mock_self.helper.read_cr.return_value = test_value
        result = CPU.read_cr(mock_self, 0x0, 0x0)
        self.assertEqual(result, test_value)

    def test_hal_cpu_read_cr_cmd(self):
        mock_self = Mock()
        test_value = 0x8086
        mock_self.helper.read_cr.return_value = test_value
        CPU.read_cr(mock_self, 0x16, 0x4)
        mock_self.helper.read_cr.assert_called_with(0x16, 0x4)

    def test_hal_cpu_write_cr(self):
        mock_self = Mock()
        expected_result = 0x01
        mock_self.helper.write_cr.return_value = expected_result
        result = CPU.write_cr(mock_self, 0x0, 0x0, expected_result)
        self.assertEqual(result, expected_result)

    def test_hal_cpu_write_cr_cmd(self):
        mock_self = Mock()
        mock_self.helper.write_cr.return_value = 0x1
        CPU.write_cr(mock_self, 0x32, 0x9, 0xAFAFAFAF)
        mock_self.helper.write_cr.assert_called_with(0x32, 0x9, 0xAFAFAFAF)

    def test_hal_cpu_write_cr_mock(self):
        mock_self = Mock()
        test_value = 0x1
        mock_self.helper.write_cr.return_value = test_value
        result = CPU.write_cr(mock_self, 0x0, 0x0, 0x0)
        self.assertEqual(result, test_value)

    def test_hal_cpuid(self):
        mock_self = Mock()
        expected_result = (0x80, 0x70, 0x60, 0x50)
        mock_self.helper.cpuid.return_value = expected_result
        result = CPU.cpuid(mock_self, 0x9, 0x4)
        self.assertEqual(result, expected_result)

    def test_hal_cpuid_cmd(self):
        mock_self = Mock()
        mock_self.helper.cpuid.return_value = (0x90, 0x80, 0x70, 0x60)
        CPU.cpuid(mock_self, 0x4000, 0x9000)
        mock_self.helper.cpuid.assert_called_with(0x4000, 0x9000)

    def test_hal_cpu_check_vmm_vmm_none(self):
        mock_self = Mock()
        VMM_NONE = 0x0
        expected_value = VMM_NONE
        mock_self.cpuid.return_value = (0, 0x3100800, 0x7FFAFBFF, 0xBFEBFBFF)
        result = CPU.check_vmm(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_check_vmm_cmd(self):
        mock_self = Mock()
        mock_self.cpuid.return_value = (0, 0x3100900, 0x7FFAFCFF, 0xBFEBFCFF)
        CPU.check_vmm(mock_self)
        mock_self.cpuid.assert_called_with(0x1, 0x0)

    def test_hal_cpu_check_vmm_vmm_none_vmm(self):
        mock_self = Mock()
        VMM_NONE = 0x0
        expected_value = VMM_NONE
        mock_self.cpuid.side_effect = [(0, 0x3100800, 0x80000000, 0xBFEBFBFF),
                                       (0, 0x566E6558, 0x666F736F, 0x4D4D566E)]
        result = CPU.check_vmm(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_check_vmm_vmm_xen(self):
        mock_self = Mock()
        VMM_XEN = 0x1
        expected_value = VMM_XEN
        mock_self.cpuid.side_effect = [(0, 0x3100800, 0x80000000, 0xBFEBFBFF),
                                       (0, 0x566E6558, 0x65584D4D, 0x4D4D566E)]
        result = CPU.check_vmm(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_check_vmm_vmm_hyper_v(self):
        mock_self = Mock()
        VMM_HYPER_V = 0x2
        expected_value = VMM_HYPER_V
        mock_self.cpuid.side_effect = [(0, 0x3100800, 0x80000000, 0xBFEBFBFF),
                                       (0, 0x7263694D, 0x666F736F, 0x76482074)]
        result = CPU.check_vmm(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_check_vmm_vmm_vmware(self):
        mock_self = Mock()
        VMM_VMWARE = 0x3
        expected_value = VMM_VMWARE
        mock_self.cpuid.side_effect = [(0, 0x3100800, 0x80000000, 0xBFEBFBFF),
                                       (0, 0x61774D56, 0x4D566572, 0x65726177)]
        result = CPU.check_vmm(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_check_vmm_vmm_kvm(self):
        mock_self = Mock()
        VMM_KVM = 0x4
        expected_value = VMM_KVM
        mock_self.cpuid.side_effect = [(0, 0x3100800, 0x80000000, 0xBFEBFBFF),
                                       (0, 0x4B4D564B, 0x564B4D56, 0x0000004D)]
        result = CPU.check_vmm(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_is_HT_active_false(self):
        mock_self = Mock()
        mock_self.get_number_logical_processor_per_core.return_value = 1
        result = CPU.is_HT_active(mock_self)
        self.assertFalse(result)

    def test_hal_cpu_is_HT_active_true(self):
        mock_self = Mock()
        mock_self.get_number_logical_processor_per_core.return_value = 4
        result = CPU.is_HT_active(mock_self)
        self.assertTrue(result)

    def test_hal_cpu_get_number_logical_processor_per_core_cmd(self):
        mock_self = Mock()
        mock_self.cpuid.return_value = (0x32, 0x16, 0x8, 0x4)
        CPU.get_number_logical_processor_per_core(mock_self)
        mock_self.cpuid.assert_called_with(0xb, 0x0)

    def test_hal_cpu_get_number_logical_processor_per_core(self):
        mock_self = Mock()
        expected_value = 4
        mock_self.cpuid.return_value = (0, expected_value, 0, 0)
        result = CPU.get_number_logical_processor_per_core(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_get_number_logical_processor_per_package(self):
        mock_self = Mock()
        expected_value = 16
        mock_self.cpuid.return_value = (0, expected_value, 0, 0)
        result = CPU.get_number_logical_processor_per_package(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_get_number_logical_processor_per_package_cmd(self):
        mock_self = Mock()
        mock_self.cpuid.return_value = (0x8000000000000000, 0x7000000000000000, 0x6000000000000000, 0x5000000000000000)
        CPU.get_number_logical_processor_per_package(mock_self)
        mock_self.cpuid.assert_called_with(0xb, 0x1)

    def test_hal_cpu_get_number_physical_processor_per_package(self):
        mock_self = Mock()
        core = 4
        package = 16
        expected_value = package // core
        mock_self.get_number_logical_processor_per_core.return_value = core
        mock_self.get_number_logical_processor_per_package.return_value = package
        result = CPU.get_number_physical_processor_per_package(mock_self)
        self.assertEqual(result, expected_value)

    def test_hal_cpu_get_cpu_topology(self):
        mock_self = Mock()
        mock_self.cs.helper.get_threads_count.return_value = 4
        mock_self.cs.cpu.cpuid.side_effect = [(4, 0, 0, 0),
                                              (1, 0, 0, 0),
                                              (4, 0, 0, 2),
                                              (1, 0, 0, 2),
                                              (4, 0, 0, 1),
                                              (1, 0, 0, 1),
                                              (4, 0, 0, 3),
                                              (1, 0, 0, 3),
                                              ]
        expected_result = {'cores': {0: [0, 2], 1: [1, 3]}, 'packages': {0: [0, 1, 2, 3]}}
        result = CPU.get_cpu_topology(mock_self)
        self.assertEqual(result, expected_result)

    def test_hal_cpu_get_number_sockets_from_APIC_table(self):
        mock_self = Mock()
        mock_self.get_number_threads_from_APIC_table.return_value = 4
        mock_self.get_number_logical_processor_per_package.return_value = 2
        result = CPU.get_number_sockets_from_APIC_table(mock_self)
        self.assertEqual(result, 2)

    def test_hal_cpu_get_SMRR(self):
        mock_self = Mock()
        mock_self.cs.register.read_field.side_effect = [0x88400000, 0xFFC00000]
        result = CPU.get_SMRR(mock_self)
        self.assertEqual(result, (0x88400000, 0xFFC00000))

    def test_hal_cpu_get_SMRR_cmd(self):
        mock_self = Mock()
        mock_self.cs.register.read_field.side_effect = [0x88400000, 0xFFC00000]
        call_1 = call('IA32_SMRR_PHYSBASE', 'PhysBase', True)
        call_2 = call('IA32_SMRR_PHYSMASK', 'PhysMask', True)
        CPU.get_SMRR(mock_self)
        mock_self.cs.register.read_field.assert_has_calls([call_1, call_2])

    def test_hal_cpu_get_SMRR_SMRAM(self):
        mock_self = Mock()
        mock_self.get_SMRR.return_value = (0x88400000, 0xFFC00000)
        expected_result = (0x88400000, 0x887FFFFF, 0x400000)
        result = CPU.get_SMRR_SMRAM(mock_self)
        self.assertEqual(result, expected_result)

    def test_hal_cpu_get_TSEG_is_server_true_cmd(self):
        mock_self = Mock()
        call_1 = call('TSEG_BASE', 'base', preserve_field_position=True)
        call_2 = call('TSEG_LIMIT', 'limit', preserve_field_position=True)
        mock_self.cs.is_server.return_value = True
        mock_self.cs.register.read_field.side_effect = [0x88400000, 0x88700000]
        CPU.get_TSEG(mock_self)
        mock_self.cs.register.read_field.assert_has_calls([call_1, call_2])

    def test_hal_cpu_get_TSEG_is_server_true(self):
        mock_self = Mock()
        mock_self.cs.is_server.return_value = True
        mock_self.cs.register.read_field.side_effect = [0x88400000, 0x88700000]
        expected_result = (0x88400000, 0x887FFFFF, 0x400000)
        result = CPU.get_TSEG(mock_self)
        self.assertEqual(result, expected_result)

    def test_hal_cpu_get_TSEG_is_server_false(self):
        mock_self = Mock()
        mock_self.cs.is_server.return_value = False
        mock_self.cs.register.read_field.side_effect = [0x88400000, 0x88800000]
        expected_result = (0x88400000, 0x887FFFFF, 0x400000)
        result = CPU.get_TSEG(mock_self)
        self.assertEqual(result, expected_result)

    def test_hal_cpu_get_TSEG_is_server_false_cmd(self):
        mock_self = Mock()
        call_1 = call('PCI0.0.0_TSEGMB', 'TSEGMB', preserve_field_position=True)
        call_2 = call('PCI0.0.0_BGSM', 'BGSM', preserve_field_position=True)
        mock_self.cs.is_server.return_value = False
        mock_self.cs.register.read_field.side_effect = [0x88400000, 0x88800000]
        CPU.get_TSEG(mock_self)
        mock_self.cs.register.read_field.assert_has_calls([call_1, call_2])

    def test_hal_cpu_get_SMRAM_smrr_true(self):
        mock_self = MagicMock()
        mock_self.check_SMRR_supported.__bool__.return_value = True
        expected_result = (0x88400000, 0x88400000, 0x400000)
        mock_self.get_SMRR_SMRAM.return_value = expected_result
        result = CPU.get_SMRAM(mock_self)
        self.assertEqual(result, expected_result)

    def test_hal_cpu_get_SMRAM_smrr_false(self):
        mock_self = MagicMock()
        mock_self.check_SMRR_supported.__bool__.return_value = False
        expected_result = (0x88400000, 0x88400000, 0x400000)
        mock_self.get_TSEG.return_value = expected_result
        result = CPU.get_SMRAM(mock_self)
        self.assertEqual(result, expected_result)

    def test_hal_cpu_check_SMRR_supported_cmd2(self):
        mock_self = Mock()
        mock_self.cs.register.read.return_value = 0x88442200
        mock_self.cs.register.get_field.return_value = 0
        CPU.check_SMRR_supported(mock_self)
        mock_self.cs.register.read.assert_called_with('MTRRCAP')

    def test_hal_cpu_check_SMRR_supported_cmd(self):
        mock_self = Mock()
        mock_self.cs.register.read.return_value = 0x88442200
        mock_self.cs.register.get_field.return_value = 0
        CPU.check_SMRR_supported(mock_self)
        mock_self.cs.register.get_field.assert_called_with('MTRRCAP', 0x88442200, 'SMRR')

    def test_hal_cpu_check_SMRR_supported_false(self):
        mock_self = Mock()
        mock_self.cs.register.get_field.return_value = 0
        result = CPU.check_SMRR_supported(mock_self)
        self.assertFalse(result)

    def test_hal_cpu_check_SMRR_supported_true(self):
        mock_self = Mock()
        mock_self.cs.register.get_field.return_value = 1
        result = CPU.check_SMRR_supported(mock_self)
        self.assertTrue(result)

    @patch('chipsec.hal.cpu.logger', spec=True)
    @patch('chipsec.hal.cpu.paging', spec=True)
    def test_hal_cpu_dump_page_tables_none(self, mock_paging, mock_logger):
        mock_self = Mock()
        mock_paging.c_ia32e_page_tables.return_value = Mock(failure=False)
        mock_logger().log_error.return_value = Mock()
        CPU.dump_page_tables(mock_self, 0x16B334001, None)
        self.assertEqual(5, mock_logger.call_count)

    @patch('chipsec.hal.cpu.logger', spec=True)
    @patch('chipsec.hal.cpu.paging.c_ia32e_page_tables', spec=True)
    def test_hal_cpu_dump_page_tables(self, mock_paging, mock_logger):
        mock_self = Mock()
        mock_paging.return_value = Mock(failure=False)
        CPU.dump_page_tables(mock_self, 0x16B334001, 'cpu0_pt_16B334001')
        self.assertEqual(4, mock_logger.call_count)

    @patch('chipsec.hal.cpu.logger', spec=True)
    @patch('chipsec.hal.cpu.paging.c_ia32e_page_tables', spec=True)
    def test_hal_cpu_dump_page_tables_log_error(self, mock_paging, mock_logger):
        mock_self = Mock()
        mock_paging.return_value = Mock(failure=True)
        CPU.dump_page_tables(mock_self, 0x16B334001, 'cpu0_pt_16B334001')
        self.assertEqual(5, mock_logger.call_count)

    def test_hal_cpu_dump_page_tables_all_1_thread(self):
        mock_self = Mock()
        threads = 1
        mock_self.cs.msr.get_cpu_thread_count.return_value = threads
        mock_self.return_value.dump_page_tables.return_value = 1
        mock_self.read_cr.side_effect = [0]
        CPU.dump_page_tables_all(mock_self)
        self.assertEqual(mock_self.dump_page_tables.call_count, threads)

    def test_hal_cpu_dump_page_tables_all_4_threads(self):
        mock_self = Mock()
        threads = 4
        mock_self.cs.msr.get_cpu_thread_count.return_value = threads
        mock_self.return_value.dump_page_tables.return_value = 1
        mock_self.read_cr.side_effect = [0, 1, 2, 3]
        CPU.dump_page_tables_all(mock_self)
        self.assertEqual(mock_self.dump_page_tables.call_count, threads)


if __name__ == '__main__':
    unittest.main()



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2022, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Common include file for modules

"""

import chipsec.chipset
from chipsec.library.logger import logger
from chipsec.library.returncode import ModuleResult, ReturnCode, result_priority


class BaseModule:
    def __init__(self):
        self.cs = chipsec.chipset.cs()
        self.logger = logger()
        self.result = ReturnCode(self.cs)
        # -------------------------------------------------------
        # Legacy results
        # -------------------------------------------------------
        self.res = ModuleResult.PASSED
        # -------------------------------------------------------

    def is_supported(self) -> bool:
        """
        This method should be overwritten by the module returning True or False
        depending whether or not this module is supported in the currently running
        platform.
        To access the currently running platform use
        """
        return True

    # -------------------------------------------------------
    # Legacy results
    # -------------------------------------------------------
    def update_res(self, value) -> None:
        if value not in result_priority:
            self.logger.log_verbose(f'Attempting to set invalid result status: {value}')
            return
        cur_priority = result_priority[self.res]
        new_priority = result_priority[value]
        if new_priority >= cur_priority:
            self.res = value
    # -------------------------------------------------------

    def run(self, module_argv) -> int:
        """
        This method should be overwritten by the module returning int
        """
        raise NotImplementedError('Sub-class should overwrite the run() method')


BIOS = 'BIOS'
SMM = 'SMM'
SECUREBOOT = 'SECUREBOOT'
HWCONFIG = 'HWCONFIG'
CPU = 'CPU'
ARCHIVED = 'ARCHIVED'
IA = 'IA'
AMD = 'AMD'


# ! [Available Tags]
MODULE_TAG_METAS = {
    BIOS: 'System Firmware (BIOS/UEFI) Modules',
    SMM: 'System Management Mode (SMM) Modules',
    SECUREBOOT: 'Secure Boot Modules',
    HWCONFIG: 'Hardware Configuration Modules',
    CPU: 'CPU Modules',
    ARCHIVED: 'Archived Modules',
    IA: 'Intel Architecture Modules',
    AMD: 'AMD Modules'
}
# ! [Available Tags]
MODULE_TAGS = dict([(_tag, []) for _tag in MODULE_TAG_METAS])

OPT_MODIFY = 'modify'



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
This module checks current contents of UEFI firmware ROM or specified firmware image for blocked EFI binaries
which can be EFI firmware volumes, EFI executable binaries (PEI modules, DXE drivers..) or EFI sections.
The module can find EFI binaries by their UI names, EFI GUIDs, MD5/SHA-1/SHA-256 hashes
or contents matching specified regular expressions.

Important! This module can only detect what it knows about from its config file.
If a bad or vulnerable binary is not detected then its 'signature' needs to be added to the config.

Usage:
  ``chipsec_main.py -i -m tools.uefi.scan_blocked [-a <fw_image>,<blockedlist>]``
    - ``fw_image``	Full file path to UEFI firmware image. If not specified, the module will dump firmware image directly from ROM
    - ``blockedlist``	JSON file with configuration of blocked EFI binaries (default = ``blockedlist.json``). Config file should be located in the same directory as this module

Examples:

    >>> chipsec_main.py -m tools.uefi.scan_blocked

Dumps UEFI firmware image from flash memory device, decodes it and checks for blocked EFI modules defined in the default config ``blockedlist.json``

    >>> chipsec_main.py -i --no_driver -m tools.uefi.scan_blocked -a uefi.rom,blockedlist.json

Decodes ``uefi.rom`` binary with UEFI firmware image and checks for blocked EFI modules defined in ``blockedlist.json`` config

.. note::
    - ``-i`` and ``--no_driver`` arguments can be used in this case because the test does not depend on the platform
      and no kernel driver is required when firmware image is specified

"""

import json
import os

from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.hal.spi_uefi import search_efi_tree, build_efi_model, EFIModuleType
from chipsec.hal.uefi import UEFI
from chipsec.hal.spi import SPI, BIOS
from chipsec.hal.uefi_search import check_match_criteria
from chipsec.library.file import read_file, get_main_dir

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'UEFI', 'SCAN_BLOCKED']

DEF_FWIMAGE_FILE = 'fw.bin'


class scan_blocked(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

        self.uefi = UEFI(self.cs)
        self.cfg_name = 'blockedlist.json'
        self.image = None
        self.efi_blockedlist = None
        self.cpuid = None

    def is_supported(self):
        return True

    def blockedlist_callback(self, efi_module):
        return check_match_criteria(efi_module, self.efi_blockedlist, self.logger, self.cpuid)

    def check_blockedlist(self):
        res = ModuleResult.PASSED

        self.logger.log(f'[*] Searching for EFI binaries that match criteria from \'{self.cfg_name}\':')
        for k in self.efi_blockedlist.keys():
            entry = self.efi_blockedlist[k]
            self.logger.log(f'    {k:16} - {entry["description"] if "description" in entry else ""}')

        # parse the UEFI firmware image and look for EFI modules matching the block-list
        efi_tree = build_efi_model(self.image, None)

        match_types = EFIModuleType.SECTION_EXE
        matching_modules = search_efi_tree(efi_tree, self.blockedlist_callback, match_types)
        found = len(matching_modules) > 0
        self.logger.log('')
        if found:
            res = ModuleResult.WARNING
            self.result.setStatusBit(self.result.status.VERIFY)
            self.logger.log_warning("Blocked EFI binary found in the UEFI firmware image")
        else:
            self.logger.log_passed("Didn't find any blocked EFI binary")
        return res

    def usage(self):
        self.logger.log(__doc__.replace('`', ''))

    # --------------------------------------------------------------------------
    # run( module_argv )
    # Required function: run here all tests from this module
    # --------------------------------------------------------------------------

    def run(self, module_argv):
        self.logger.start_test("Check for blocked EFI binaries in UEFI firmware")

        self.usage()

        image_file = DEF_FWIMAGE_FILE
        if len(module_argv) == 0:
            # Read firmware image directly from SPI flash memory
            self.spi = SPI(self.cs)
            (base, limit, _) = self.spi.get_SPI_region(BIOS)
            image_size = limit + 1 - base
            self.logger.log(f'[*] Dumping FW image from ROM to {image_file}: 0x{base:08X} bytes at [0x{limit:08X}:0x{image_size:08X}]')
            self.logger.log("[*] This may take a few minutes (instead, use 'chipsec_util spi dump')...")
            self.spi.read_spi_to_file(base, image_size, image_file)
            self.cpuid = self.cs.get_cpuid()
        elif len(module_argv) > 0:
            # Use provided firmware image
            image_file = module_argv[0]
            self.logger.log(f'[*] Reading FW image from file: {image_file}')

        self.image = read_file(image_file)

        if not self.image:
            if len(module_argv) == 0:
                self.logger.log_important('Unable to read SPI and generate FW image. Access may be blocked.')
            self.logger.log_error('No FW image file to read.  Exiting!')
            self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
            return self.result.getReturnCode(ModuleResult.ERROR)

        # Load JSON config with blocked EFI modules
        if len(module_argv) > 1:
            self.cfg_name = module_argv[1]
        cfg_pth = os.path.join(get_main_dir(), "chipsec/modules/tools/uefi", self.cfg_name)
        with open(cfg_pth, 'r') as blockedlist_json:
            self.efi_blockedlist = json.load(blockedlist_json)

        self.res = self.check_blockedlist()
        return self.result.getReturnCode(self.res)




# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2019-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
HAL component providing access to and decoding of SMBIOS structures
"""

import struct
from collections import namedtuple
from typing import Dict, List, Optional, Tuple, Any, Union, Type
from chipsec.library.defines import BOUNDARY_1MB, bytestostring
from chipsec.hal import hal_base, uefi
from chipsec.library.logger import logger

SCAN_LOW_LIMIT = 0xF0000
SCAN_SIZE = 0x10000

SMBIOS_2_x_SIG = b"_SM_"
SMBIOS_2_x_ENTRY_SIZE = 0x1F
SMBIOS_2_x_ENTRY_SIZE_OLD = 0x1E
SMBIOS_2_x_MAJOR_VER = 0x02
SMBIOS_2_x_INT_SIG = b"_DMI_"
SMBIOS_2_x_GUID = "EB9D2D31-2D88-11D3-9A16-0090273FC14D"
SMBIOS_2_x_ENTRY_POINT_FMT = "=4sBBBBHB5B5sBHIHB"
SMBIOS_2_x_ENTRY_POINT_SIZE = struct.calcsize(SMBIOS_2_x_ENTRY_POINT_FMT)


class SMBIOS_2_x_ENTRY_POINT(namedtuple('SMBIOS_2_x_ENTRY_POINT', 'Anchor EntryCs EntryLen MajorVer MinorVer MaxSize EntryRev \
        FormatArea0 FormatArea1 FormatArea2 FormatArea3 FormatArea4 IntAnchor IntCs TableLen TableAddr NumStructures BcdRev')):
    __slots__ = ()

    def __str__(self) -> str:
        return f"""
SMBIOS 2.x Entry Point Structure:
  Anchor String             : {bytestostring(self.Anchor)}
  Checksum                  : 0x{self.EntryCs:02X}
  Entry Point Length        : 0x{self.EntryLen:02X}
  Entry Point Version       : {self.MajorVer:d}.{self.MinorVer:d}
  Max Structure Size        : 0x{self.MaxSize:04X}
  Entry Point Revision      : 0x{self.EntryRev:02X}
  Formatted Area            : 0x{self.FormatArea0:02X}, 0x{self.FormatArea1:02X}, 0x{self.FormatArea2:02X}, 0x{self.FormatArea3:02X}, 0x{self.FormatArea4:02X}
  Intermediate Anchor String: {bytestostring(self.IntAnchor)}
  Intermediate Checksum     : 0x{self.IntCs:02X}
  Structure Table Length    : 0x{self.TableLen:04X}
  Structure Table Address   : 0x{self.TableAddr:08X}
  SMBIOS Structure Count    : 0x{self.NumStructures:04X}
  SMBIOS BCD Revision       : 0x{self.BcdRev:02X}
"""


SMBIOS_3_x_SIG = b"_SM3_"
SMBIOS_3_x_ENTRY_SIZE = 0x18
SMBIOS_3_x_MAJOR_VER = 0x03
SMBIOS_3_x_GUID = "F2FD1544-9794-4A2C-992E-E5BBCF20E394"
SMBIOS_3_x_ENTRY_POINT_FMT = "=5sBBBBBBBIQ"
SMBIOS_3_x_ENTRY_POINT_SIZE = struct.calcsize(SMBIOS_3_x_ENTRY_POINT_FMT)


class SMBIOS_3_x_ENTRY_POINT(namedtuple('SMBIOS_3_x_ENTRY_POINT', 'Anchor EntryCs EntryLen MajorVer MinorVer Docrev EntryRev \
        Reserved MaxSize TableAddr')):
    __slots__ = ()

    def __str__(self) -> str:
        return f"""
SMBIOS 3.x Entry Point Structure:
  Anchor String             : {bytestostring(self.Anchor)}
  Checksum                  : 0x{self.EntryCs:02X}
  Entry Point Length        : 0x{self.EntryLen:02X}
  Entry Ponnt Version       : {self.MajorVer:d}.{self.MinorVer:d}
  SMBIOS Docrev             : 0x{self.Docrev:02X}
  Entry Point Revision      : 0x{self.EntryRev:02X}
  Reserved                  : 0x{self.Reserved:02X}
  Max Structure Size        : 0x{self.MaxSize:08X}
  Structure Table Address   : 0x{self.TableAddr:016X}
"""


SMBIOS_STRUCT_HEADER_FMT = "=BBH"
SMBIOS_STRUCT_HEADER_SIZE = struct.calcsize(SMBIOS_STRUCT_HEADER_FMT)


class SMBIOS_STRUCT_HEADER(namedtuple('SMBIOS_STRUCT_HEADER', 'Type Length Handle')):
    __slots__ = ()

    def __str__(self) -> str:
        return f"""
SMBIOS Struct Header:
  Type                      : 0x{self.Type:02X} ({self.Type:d})
  Length                    : 0x{self.Length:02X}
  Handle                    : 0x{self.Handle:04X}
"""


SMBIOS_STRUCT_TERM_FMT = "=H"
SMBIOS_STRUCT_TERM_SIZE = struct.calcsize(SMBIOS_STRUCT_TERM_FMT)
SMBIOS_STRUCT_TERM_VAL = 0x0000


SMBIOS_BIOS_INFO_ENTRY_ID = 0
SMBIOS_BIOS_INFO_2_0_ENTRY_FMT = '=BBHBBHBBQ'
SMBIOS_BIOS_INFO_2_0_ENTRY_SIZE = struct.calcsize(SMBIOS_BIOS_INFO_2_0_ENTRY_FMT)
SMBIOS_BIOS_INFO_2_0_FORMAT_STRING_FAILED = """
SMBIOS BIOS Information structure decode failed
"""


class SMBIOS_BIOS_INFO_2_0(namedtuple('SMBIOS_BIOS_INFO_2_0_ENTRY', 'type length handle vendor_str version_str segment \
        release_str rom_sz bios_char strings')):
    __slots__ = ()

    def __str__(self) -> str:
        str_count = len(self.strings)
        ven_str = ''
        ver_str = ''
        rel_str = ''
        if self.vendor_str != 0 and self.vendor_str <= str_count:
            ven_str = self.strings[self.vendor_str - 1]
        if self.version_str != 0 and self.version_str <= str_count:
            ver_str = self.strings[self.version_str - 1]
        if self.release_str != 0 and self.release_str <= str_count:
            rel_str = self.strings[self.release_str - 1]
        return f"""
SMBIOS BIOS Information:
  Type                      : 0x{self.type:02X} ({self.type:d})
  Length                    : 0x{self.length:02X}
  Handle                    : 0x{self.handle:04X}
  Vendor                    : {ven_str:s}
  BIOS Version              : {ver_str:s}
  BIOS Starting Segment     : 0x{self.segment:04X}
  BIOS Release Date         : {rel_str:s}
  BIOS ROM Size             : 0x{self.rom_sz:02X}
  BIOS Characteristics      : 0x{self.bios_char:016X}
"""


SMBIOS_SYSTEM_INFO_ENTRY_ID = 1
SMBIOS_SYSTEM_INFO_2_0_ENTRY_FMT = '=BBHBBBB'
SMBIOS_SYSTEM_INFO_2_0_ENTRY_SIZE = struct.calcsize(SMBIOS_SYSTEM_INFO_2_0_ENTRY_FMT)
SMBIOS_SYSTEM_INFO_2_0_FORMAT_STRING_FAILED = """
SMBIOS System Information structure decode failed
"""


class SMBIOS_SYSTEM_INFO_2_0(namedtuple('SMBIOS_SYSTEM_INFO_2_0_ENTRY', 'type length handle manufacturer_str product_str \
        version_str serial_str strings')):
    __slots__ = ()

    def __str__(self) -> str:
        str_count = len(self.strings)
        man_str = ''
        pro_str = ''
        ver_str = ''
        ser_str = ''
        if self.manufacturer_str != 0 and self.manufacturer_str <= str_count:
            man_str = self.strings[self.manufacturer_str - 1]
        if self.product_str != 0 and self.product_str <= str_count:
            pro_str = self.strings[self.product_str - 1]
        if self.version_str != 0 and self.version_str <= str_count:
            ver_str = self.strings[self.version_str - 1]
        if self.serial_str != 0 and self.serial_str <= str_count:
            ser_str = self.strings[self.serial_str - 1]
        return f"""
SMBIOS System Information:
  Type                      : 0x{self.type:02X} ({self.type:d})
  Length                    : 0x{self.length:02X}
  Handle                    : 0x{self.handle:04X}
  Manufacturer              : {man_str:s}
  Product Name              : {pro_str:s}
  Version                   : {ver_str:s}
  Serial Number             : {ser_str:s}
"""


SmbiosInfo = Union[SMBIOS_BIOS_INFO_2_0, SMBIOS_SYSTEM_INFO_2_0]
StructDecode = Dict[str, Any]  # TODO: Replace Any when TypeDict (PEP 589) supported

struct_decode_tree: Dict[int, StructDecode] = {
    SMBIOS_BIOS_INFO_ENTRY_ID: {'class': SMBIOS_BIOS_INFO_2_0, 'format': SMBIOS_BIOS_INFO_2_0_ENTRY_FMT},
    SMBIOS_SYSTEM_INFO_ENTRY_ID: {'class': SMBIOS_SYSTEM_INFO_2_0, 'format': SMBIOS_SYSTEM_INFO_2_0_ENTRY_FMT}
}


class SMBIOS(hal_base.HALBase):
    def __init__(self, cs):
        super(SMBIOS, self).__init__(cs)
        self.uefi = uefi.UEFI(cs)
        self.smbios_2_guid_found = False
        self.smbios_2_pa = None
        self.smbios_2_ep = None
        self.smbios_2_data = None
        self.smbios_3_guid_found = False
        self.smbios_3_pa = None
        self.smbios_3_ep = None
        self.smbios_3_data = None

    def __get_raw_struct(self, table: bytes, start_offset: int) -> Tuple[Optional[bytes], Optional[int]]:
        """
        Returns a tuple including the raw data and the offset to the next entry.  This allows the function
        to be called multiple times to process all the entries in a table.

        Return Value:
        (raw_data, next_offset)

        Error/End:
        (None, None)
        """
        # Check for end of table and remaining size to parse
        if table is None:
            logger().log_hal('- Invalid table')
            return (None, None)
        table_len = len(table)
        logger().log_hal(f'Start Offset: 0x{start_offset:04X}, Table Size: 0x{table_len:04X}')
        if start_offset >= table_len:
            logger().log_hal(f'- Bad table length (table_len): 0x{table_len:04X}')
            return (None, None)
        size_left = len(table[start_offset:])
        if size_left < SMBIOS_STRUCT_HEADER_SIZE:
            logger().log_hal(f'- Table too small (size_left): 0x{size_left:04X}')
            return (None, None)

        # Read the header to determine structure fixed size
        try:
            header = SMBIOS_STRUCT_HEADER(*struct.unpack_from(SMBIOS_STRUCT_HEADER_FMT,
                                                              table[start_offset:start_offset + SMBIOS_STRUCT_HEADER_SIZE]))
        except:
            logger().log_hal('- Unable to unpack data')
            return (None, None)
        str_offset = start_offset + header.Length
        if str_offset + SMBIOS_STRUCT_TERM_SIZE >= table_len:
            logger().log_hal(f'- Not enough space for termination (str_offset): 0x{str_offset:04X}')
            return (None, None)

        # Process any remaining content (strings)
        logger().log_hal(f'String start offset: 0x{str_offset:04X}')
        tmp_offset = str_offset
        while (tmp_offset + SMBIOS_STRUCT_TERM_SIZE < table_len):
            (value, ) = struct.unpack_from(SMBIOS_STRUCT_TERM_FMT, table[tmp_offset:tmp_offset + SMBIOS_STRUCT_TERM_SIZE])
            if value == SMBIOS_STRUCT_TERM_VAL:
                logger().log_hal('+ Found structure termination')
                break
            tmp_offset += 1
        if tmp_offset >= table_len:
            logger().log_hal('- End of table reached')
            return (None, None)
        tmp_offset += SMBIOS_STRUCT_TERM_SIZE

        logger().log_hal(f'Structure Size: 0x{tmp_offset - start_offset:04X}')
        return (table[start_offset:tmp_offset], tmp_offset)

    def __validate_ep_2_values(self, pa: int) -> Optional[SMBIOS_2_x_ENTRY_POINT]:
        # Force a second read of memory so we don't have to worry about it falling outside the
        # original buffer.
        try:
            logger().log_hal(f'Validating 32bit SMBIOS header @ 0x{pa:08X}')
            mem_buffer = self.cs.mem.read_physical_mem(pa, SMBIOS_2_x_ENTRY_POINT_SIZE)
            ep_data = SMBIOS_2_x_ENTRY_POINT(*struct.unpack_from(SMBIOS_2_x_ENTRY_POINT_FMT, mem_buffer))
        except:
            logger().log_hal('- Memory read failed')
            return None
        if ep_data.Anchor != SMBIOS_2_x_SIG:
            logger().log_hal('- Invalid signature')
            return None
        if not (ep_data.EntryLen == SMBIOS_2_x_ENTRY_SIZE or ep_data.EntryLen == SMBIOS_2_x_ENTRY_SIZE_OLD):
            logger().log_hal('- Invalid structure size')
            return None
        if ep_data.IntAnchor != SMBIOS_2_x_INT_SIG:
            logger().log_hal('- Invalid intermediate signature')
            return None
        if (ep_data.TableAddr == 0) or (ep_data.TableLen == 0):
            logger().log_hal('- Invalid table address or length')
            return None
        return ep_data

    def __validate_ep_3_values(self, pa: int) -> Optional[SMBIOS_3_x_ENTRY_POINT]:
        # Force a second read of memory so we don't have to worry about it falling outside the
        # original buffer.
        try:
            logger().log_hal(f'Validating 64bit SMBIOS header @ 0x{pa:08X}')
            mem_buffer = self.cs.mem.read_physical_mem(pa, SMBIOS_3_x_ENTRY_POINT_SIZE)
            ep_data = SMBIOS_3_x_ENTRY_POINT(*struct.unpack_from(SMBIOS_3_x_ENTRY_POINT_FMT, mem_buffer))
        except:
            logger().log_hal('- Memory read failed')
            return None
        if ep_data.Anchor != SMBIOS_3_x_SIG:
            logger().log_hal('- Invalid signature')
            return None
        if not (ep_data.EntryLen == SMBIOS_3_x_ENTRY_SIZE):
            logger().log_hal('- Invalid structure size')
            return None
        if ep_data.MaxSize == 0 or ep_data.TableAddr == 0:
            logger().log_hal('- Invalid table address or maximum size')
            return None
        return ep_data

    def find_smbios_table(self) -> bool:
        # Handle the case were we already found the tables
        if self.smbios_2_ep is not None or self.smbios_3_ep is not None:
            return True

        # Initialize search parameters
        entries_to_find = entries_found = 0

        # Fist get the configuration table using the UEFI HAL.  You may not be able to use the addresses
        # in the table because in some cases they have been converted to a VA and are not mapped.
        logger().log_hal('Checking UEFI Configuration Table for SMBIOS entry')
        (ect_found, _, ect, _) = self.uefi.find_EFI_Configuration_Table()
        if ect_found and (ect is not None):
            logger().log_hal(str(ect))
            if SMBIOS_2_x_GUID in ect.VendorTables:
                logger().log_hal('+ Found 32bit SMBIOS entry')
                logger().log_hal(f'+ Potential 2.x table address: 0x{ect.VendorTables[SMBIOS_2_x_GUID]:016X}')
                self.smbios_2_guid_found = True
                entries_to_find += 1
            if SMBIOS_3_x_GUID in ect.VendorTables:
                logger().log_hal('+ Found 64bit SMBIOS entry')
                logger().log_hal(f'+ Potential 3.x table address: 0x{ect.VendorTables[SMBIOS_3_x_GUID]:016X}')
                self.smbios_3_guid_found = True
                entries_to_find += 1

        # Determine regions to scan
        if self.smbios_2_guid_found or self.smbios_3_guid_found:
            (smm_base, _, _) = self.cs.cpu.get_SMRAM()
            pa = smm_base - SCAN_SIZE
        else:
            entries_to_find = 2
            pa = BOUNDARY_1MB - SCAN_SIZE

        # Scan memory for the signature
        logger().log_hal(f'Scanning memory for {entries_to_find:d} signature(s)')
        while (pa >= SCAN_LOW_LIMIT):
            mem_buffer = self.cs.mem.read_physical_mem(pa, SCAN_SIZE)
            sig_pa = mem_buffer.find(SMBIOS_2_x_SIG) + pa
            if sig_pa >= pa and self.smbios_2_pa is None:
                logger().log_hal(f'+ Found SMBIOS 2.x signature @ 0x{sig_pa:08X}')
                self.smbios_2_ep = self.__validate_ep_2_values(sig_pa)
                if self.smbios_2_ep is not None:
                    logger().log_hal('+ Verified SMBIOS 2.x Entry Point structure')
                    self.smbios_2_pa = sig_pa
                    entries_found += 1
            sig_pa = mem_buffer.find(SMBIOS_3_x_SIG) + pa
            if sig_pa >= pa and self.smbios_3_pa is None:
                logger().log_hal(f'+ Found SMBIOS 3.x signature @ 0x{sig_pa:08X}')
                self.smbios_3_ep = self.__validate_ep_3_values(sig_pa)
                if self.smbios_3_ep is not None:
                    logger().log_hal('+ Verified SMBIOS 3.x Entry Point structure')
                    self.smbios_3_pa = sig_pa
                    entries_found += 1
            if entries_found >= entries_to_find:
                break
            pa -= SCAN_SIZE

        # Check to see if we thing we found the structure
        if self.smbios_2_pa is None and self.smbios_3_pa is None:
            logger().log_hal('- Unable to find SMBIOS tables')
            return False

        # Read the raw data regions
        logger().log_hal('Reading SMBIOS data tables:')
        if self.smbios_2_ep is not None and self.smbios_2_ep.TableAddr != 0 and self.smbios_2_ep.TableLen != 0:
            self.smbios_2_data = self.cs.mem.read_physical_mem(self.smbios_2_ep.TableAddr, self.smbios_2_ep.TableLen)
            if self.smbios_2_data is None:
                logger().log_hal('- Failed to read 32bit SMBIOS data')
        if self.smbios_3_ep is not None and self.smbios_3_ep.TableAddr != 0 and self.smbios_3_ep.MaxSize != 0:
            self.smbios_3_data = self.cs.mem.read_physical_mem(self.smbios_3_ep.TableAddr, self.smbios_3_ep.MaxSize)
            if self.smbios_3_data is None:
                logger().log_hal('- Failed to read 64bit SMBIOS data')

        return True

    def get_raw_structs(self, struct_type: Optional[int], force_32bit: bool):
        """
        Returns a list of raw data blobs for each SMBIOS structure.  The default is to process the 64bit
        entries if available unless specifically specified.

        Error:
        None
        """
        ret_val = []

        if self.smbios_3_data is not None and not force_32bit:
            logger().log_hal('Using 64bit SMBIOS table')
            table = self.smbios_3_data
        elif self.smbios_2_data is not None:
            logger().log_hal('Using 32bit SMBIOS table')
            table = self.smbios_2_data
        else:
            logger().log_hal('- No SMBIOS data available')
            return None

        logger().log_hal('Getting SMBIOS structures...')
        raw_data, next_offset = self.__get_raw_struct(table, 0)
        while (next_offset is not None) and (raw_data is not None):
            if struct_type is None:
                ret_val.append(raw_data)
            else:
                header = SMBIOS_STRUCT_HEADER(*struct.unpack_from(SMBIOS_STRUCT_HEADER_FMT, raw_data[:SMBIOS_STRUCT_HEADER_SIZE]))
                if header is not None and header.Type == struct_type:
                    ret_val.append(raw_data)
            raw_data, next_offset = self.__get_raw_struct(table, next_offset)

        return ret_val

    def get_header(self, raw_data: bytes) -> Optional[SMBIOS_STRUCT_HEADER]:
        logger().log_hal('Getting generic SMBIOS header information')
        if raw_data is None:
            logger().log_hal('- Raw data pointer is None')
            return None
        if len(raw_data) < SMBIOS_STRUCT_HEADER_SIZE:
            logger().log_hal('- Raw data too small for header information')
            return None

        try:
            header = SMBIOS_STRUCT_HEADER(*struct.unpack_from(SMBIOS_STRUCT_HEADER_FMT, raw_data[:SMBIOS_STRUCT_HEADER_SIZE]))
        except:
            logger().log_hal('- Failed to extract information from raw data')
            return None

        return header

    def get_string_list(self, raw_data: bytes) -> Optional[List[str]]:
        ret_val = []

        logger().log_hal('Getting strings from structure')
        raw_data_size = len(raw_data)
        header = self.get_header(raw_data)
        if header is None:
            return None
        if header.Length + SMBIOS_STRUCT_TERM_SIZE > raw_data_size:
            logger().log_hal('- Data buffer too small for structure')
            return None
        if header.Length + SMBIOS_STRUCT_TERM_SIZE == raw_data_size:
            logger().log_hal('+ No strings in this structure')
            return ret_val

        index = 0
        tmp_offset = header.Length
        while tmp_offset + index + 1 < raw_data_size:
            (value, ) = struct.unpack_from('=B', raw_data[tmp_offset + index:])
            if value == 0:
                logger().log_hal(f'+ Unpacking string of size {index:d}')
                (string, ) = struct.unpack_from(f'={index:d}s', raw_data[tmp_offset:])
                string = bytestostring(string)
                logger().log_hal(f'+ Found: {string:s}')
                ret_val.append(string)
                tmp_offset += index + 1
                index = 0
                continue
            index += 1

        logger().log_hal(f'+ Found {len(ret_val):d} strings')
        return ret_val

    def get_decoded_structs(self, struct_type: Optional[int] = None, force_32bit: bool = False) -> Optional[List[Type[SmbiosInfo]]]:
        ret_val = []

        # Determine if the structure exists in the table
        logger().log_hal('Getting decoded SMBIOS structures')
        structs = self.get_raw_structs(struct_type, force_32bit)
        if structs is None:
            return None

        # Process all the entries
        for data in structs:
            # Get the structures header information so we can determine the correct decode method
            header = self.get_header(data)
            if header is None:
                logger().log_hal('- Could not decode header')
                continue
            if header.Type not in struct_decode_tree:
                logger().log_hal(f'- Structure {header.Type:d} not in decode list')
                continue

            # Unpack the structure and then get the strings
            tmp_decode = struct_decode_tree[header.Type]
            try:
                decode_data = struct.unpack_from(tmp_decode['format'], data)
            except:
                logger().log_hal('- Could not decode structure')
                continue
            if decode_data is None:
                logger().log_hal('- No structure data was decoded')
                continue
            strings = self.get_string_list(data)
            if strings is not None:
                decode_data = decode_data + (strings, )

            # Create the actual object
            try:
                decode_object = tmp_decode['class'](*decode_data)
            except:
                logger().log_hal('- Failed to create structure')
                continue
            ret_val.append(decode_object)

        return ret_val



# CHIPSEC: Platform Security Assessment Framework
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
import unittest

from chipsec.library.exceptions import UnknownChipsetError
from chipsec.config import CHIPSET_CODE_UNKNOWN
from tests.software import cs
from tests.software import mock_helper


class TestPlatformChipsecCs(cs.TestChipsecCs, unittest.TestCase):
    """Test the platform commands exposed by chipsec chipset."""

    def test_platform(self):
        p = self._chipsec_cs("get_chipset_code", mock_helper.ValidChipsetHelper)
        self.assertEqual('SKL', p)

    def test_platform_given(self):
        p = self._chipsec_cs("get_chipset_code", mock_helper.InvalidChipsetHelper, 'CML')
        self.assertEqual('CML', p)

    def test_platform_invalid(self):
        self.assertRaises(UnknownChipsetError, self._chipsec_cs, "get_chipset_code", mock_helper.InvalidChipsetHelper)
        # self.assertEqual(CHIPSET_CODE_UNKNOWN, p)

    def test_pch(self):
        p = self._chipsec_cs("get_pch_code", mock_helper.ValidChipsetHelper)
        self.assertEqual('PCH_1XX', p)

    def test_pch_invalid(self):
        p = self._chipsec_cs("get_pch_code", mock_helper.InvalidPchHelper)
        self.assertEqual(CHIPSET_CODE_UNKNOWN, p)

    def test_pch_given(self):
        p = self._chipsec_cs("get_pch_code", mock_helper.InvalidPchHelper, None, 'PCH_495')
        self.assertEqual('PCH_495', p)


if __name__ == '__main__':
    unittest.main()



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
CHIPSEC can parse an image file containing data from the SPI flash (such as the result of chipsec_util spi dump). This can be critical in forensic analysis.

This will create multiple log files, binaries, and directories that correspond to the sections, firmware volumes, files, variables, etc. stored in the SPI flash.

Usage:

    >>> chipsec_util decode <rom> [fw_type]

For a list of fw types run:

    >>> chipsec_util decode types

Examples:

    >>> chipsec_util decode spi.bin vss

.. note::
    - It may be necessary to try various options for fw_type in order to correctly parse NVRAM variables.
      Currently, CHIPSEC does not autodetect the correct format.
      If the nvram directory does not appear and the list of nvram variables is empty, try again with another type.

"""

import os
from argparse import ArgumentParser

from chipsec.library.file import read_file, write_file
from chipsec.command import BaseCommand, toLoad

from chipsec.hal.spi import FLASH_DESCRIPTOR, BIOS
from chipsec.hal.spi_descriptor import get_spi_flash_descriptor, get_spi_regions, parse_spi_flash_descriptor
from chipsec.hal.spi_uefi import decode_uefi_region
from chipsec.hal.uefi import uefi_platform


class DecodeCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.Config

    def parse_arguments(self) -> None:
        parser = ArgumentParser(usage=__doc__)
        parser.add_argument('_rom', metavar='<rom>', help='file to decode')
        parser.add_argument('_fwtype', metavar='fw_type', nargs='?', help='firmware type', default=None)
        parser.parse_args(self.argv, namespace=self)

        if self._rom.lower() == 'types':
            self.func = self.decode_types
        else:
            self.func = self.decode_rom

    def decode_types(self) -> None:
        self.logger.log(f'\n<fw_type> should be in [ {" | ".join([f"{t}" for t in uefi_platform.fw_types])} ]\n')

    def decode_rom(self) -> bool:
        self.logger.log(f'[CHIPSEC] Decoding SPI ROM image from a file \'{self._rom}\'')
        f = read_file(self._rom)
        if not f:
            return False
        (fd_off, fd) = get_spi_flash_descriptor(f)
        if (-1 == fd_off) or (fd is None):
            self.logger.log_error(f'Could not find SPI Flash descriptor in the binary \'{self._rom}\'')
            self.logger.log_information("To decode an image without a flash decriptor try chipsec_util uefi decode")
            return False

        self.logger.log(f'[CHIPSEC] Found SPI Flash descriptor at offset 0x{fd_off:X} in the binary \'{self._rom}\'')
        rom = f[fd_off:]

        # Decoding SPI Flash Regions
        flregs = get_spi_regions(fd)
        if flregs is None:
            self.logger.log_error("SPI Flash descriptor region is not valid")
            self.logger.log_information("To decode an image with an invalid flash decriptor try chipsec_util uefi decode")
            return False

        _orig_logname = self.logger.LOG_FILE_NAME

        pth = os.path.join(self.cs.os_helper.getcwd(), self._rom + ".dir")
        if not os.path.exists(pth):
            os.makedirs(pth)

        for r in flregs:
            idx = r[0]
            name = r[1]
            base = r[3]
            limit = r[4]
            notused = r[5]
            if not notused:
                region_data = rom[base:limit + 1]
                fname = os.path.join(pth, f'{idx:d}_{base:04X}-{limit:04X}_{name}.bin')
                write_file(fname, region_data)
                if FLASH_DESCRIPTOR == idx:
                    # Decoding Flash Descriptor
                    self.logger.set_log_file(os.path.join(pth, fname + '.log'), False)
                    parse_spi_flash_descriptor(self.cs, region_data)
                elif BIOS == idx:
                    # Decoding EFI Firmware Volumes
                    self.logger.set_log_file(os.path.join(pth, fname + '.log'), False)
                    decode_uefi_region(pth, fname, self._fwtype)

        self.logger.set_log_file(_orig_logname)
        return True


commands = {"decode": DecodeCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Access to IOMMU engines
"""

from chipsec.hal import hal_base, mmio, paging
from chipsec.library.exceptions import IOMMUError

IOMMU_ENGINE_DEFAULT = 'VTD'
IOMMU_ENGINE_GFX = 'GFXVTD'


IOMMU_ENGINES = {
    IOMMU_ENGINE_GFX: 'GFXVTBAR',
    IOMMU_ENGINE_DEFAULT: 'VTBAR'
}


class IOMMU(hal_base.HALBase):

    def __init__(self, cs):
        super(IOMMU, self).__init__(cs)
        self.mmio = mmio.MMIO(cs)

    def get_IOMMU_Base_Address(self, iommu_engine: str) -> int:
        if iommu_engine in IOMMU_ENGINES:
            vtd_base_name = IOMMU_ENGINES[iommu_engine]
        else:
            raise IOMMUError(f'IOMMUError: unknown IOMMU engine 0x{iommu_engine:X}')

        if self.mmio.is_MMIO_BAR_defined(vtd_base_name):
            (base, _) = self.mmio.get_MMIO_BAR_base_address(vtd_base_name)
        else:
            raise IOMMUError(f'IOMMUError: IOMMU BAR {vtd_base_name} is not defined in the config')
        return base

    def is_IOMMU_Engine_Enabled(self, iommu_engine: str) -> bool:
        if iommu_engine in IOMMU_ENGINES:
            vtd_base_name = IOMMU_ENGINES[iommu_engine]
        else:
            raise IOMMUError(f'IOMMUError: unknown IOMMU engine 0x{iommu_engine:X}')
        return self.mmio.is_MMIO_BAR_defined(vtd_base_name) and self.mmio.is_MMIO_BAR_enabled(vtd_base_name)

    def is_IOMMU_Translation_Enabled(self, iommu_engine: str) -> bool:
        tes = self.cs.register.read_field(f'{IOMMU_ENGINES[iommu_engine]}_GSTS', 'TES')
        return (1 == tes)

    def set_IOMMU_Translation(self, iommu_engine: str, te: int) -> bool:
        return self.cs.register.write_field(f'{IOMMU_ENGINES[iommu_engine]}_GCMD', 'TE', te)

    def dump_IOMMU_configuration(self, iommu_engine: str) -> None:
        self.logger.log("==================================================================")
        vtd = IOMMU_ENGINES[iommu_engine]
        self.logger.log(f'[iommu] {iommu_engine} IOMMU Engine Configuration')
        self.logger.log("==================================================================")
        self.logger.log(f'Base register (BAR)       : {vtd}')
        reg = self.cs.register.read(vtd)
        self.logger.log(f'BAR register value        : 0x{reg:X}')
        if reg == 0:
            return
        base = self.get_IOMMU_Base_Address(iommu_engine)
        self.logger.log(f'MMIO base                 : 0x{base:016X}')
        self.logger.log("------------------------------------------------------------------")
        ver_min = self.cs.register.read_field(f'{vtd}_VER', 'MIN')
        ver_max = self.cs.register.read_field(f'{vtd}_VER', 'MAX')
        self.logger.log(f'Version                   : {ver_max:X}.{ver_min:X}')
        enabled = self.is_IOMMU_Engine_Enabled(iommu_engine)
        self.logger.log(f'Engine enabled            : {enabled:d}')
        te = self.is_IOMMU_Translation_Enabled(iommu_engine)
        self.logger.log(f'Translation enabled       : {te:d}')
        rtaddr_rta = self.cs.register.read_field(f'{vtd}_RTADDR', 'RTA', True)
        self.logger.log(f'Root Table Address        : 0x{rtaddr_rta:016X}')
        irta = self.cs.register.read_field(f'{vtd}_IRTA', 'IRTA')
        self.logger.log(f'Interrupt Remapping Table : 0x{irta:016X}')
        self.logger.log("------------------------------------------------------------------")
        self.logger.log("Protected Memory:")
        pmen_epm = self.cs.register.read_field(f'{vtd}_PMEN', 'EPM')
        pmen_prs = self.cs.register.read_field(f'{vtd}_PMEN', 'PRS')
        self.logger.log(f'  Enabled                 : {pmen_epm:d}')
        self.logger.log(f'  Status                  : {pmen_prs:d}')
        plmbase = self.cs.register.read_field(f'{vtd}_PLMBASE', 'PLMB')
        plmlimit = self.cs.register.read_field(f'{vtd}_PLMLIMIT', 'PLML')
        phmbase = self.cs.register.read_field(f'{vtd}_PHMBASE', 'PHMB')
        phmlimit = self.cs.register.read_field(f'{vtd}_PHMLIMIT', 'PHML')
        self.logger.log(f'  Low Memory Base         : 0x{plmbase:016X}')
        self.logger.log(f'  Low Memory Limit        : 0x{plmlimit:016X}')
        self.logger.log(f'  High Memory Base        : 0x{phmbase:016X}')
        self.logger.log(f'  High Memory Limit       : 0x{phmlimit:016X}')
        self.logger.log("------------------------------------------------------------------")
        self.logger.log("Capabilities:\n")
        cap_reg = self.cs.register.read(f'{vtd}_CAP')
        self.cs.register.print(f'{vtd}_CAP', cap_reg)
        ecap_reg = self.cs.register.read(f'{vtd}_ECAP')
        self.cs.register.print(f'{vtd}_ECAP', ecap_reg)
        self.logger.log('')

    def dump_IOMMU_page_tables(self, iommu_engine: str) -> None:
        vtd = IOMMU_ENGINES[iommu_engine]
        if self.cs.register.read(vtd) == 0:
            self.logger.log(f'[iommu] {vtd} value is zero')
            return
        te = self.is_IOMMU_Translation_Enabled(iommu_engine)
        self.logger.log(f'[iommu] Translation enabled    : {te:d}')
        rtaddr_reg = self.cs.register.read(f'{vtd}_RTADDR')
        rtaddr_rta = self.cs.register.get_field(f'{vtd}_RTADDR', rtaddr_reg, 'RTA', True)
        rtaddr_rtt = self.cs.register.get_field(f'{vtd}_RTADDR', rtaddr_reg, 'RTT')
        self.logger.log(f'[iommu] Root Table Address/Type: 0x{rtaddr_rta:016X}/{rtaddr_rtt:X}')

        ecap_reg = self.cs.register.read(f'{vtd}_ECAP')
        ecs = self.cs.register.get_field(f'{vtd}_ECAP', ecap_reg, 'ECS')
        pasid = self.cs.register.get_field(f'{vtd}_ECAP', ecap_reg, 'PASID')
        self.logger.log(f'[iommu] PASID / ECS            : {pasid:X} / {ecs:X}')

        if 0xFFFFFFFFFFFFFFFF != rtaddr_reg:
            if te:
                self.logger.log(f'[iommu] Dumping VT-d page table hierarchy at 0x{rtaddr_rta:016X} (vtd_context_{rtaddr_rta:08X})')
                paging_vtd = paging.c_vtd_page_tables(self.cs)
                paging_vtd.read_vtd_context(f'vtd_context_{rtaddr_rta:08X}', rtaddr_rta)
                self.logger.log(f'[iommu] Total VTd domains: {len(paging_vtd.domains):d}')
                for domain in paging_vtd.domains:
                    paging_vtd.read_pt_and_show_status(f'vtd_{domain:08X}', 'VTd', domain)
            else:
                self.logger.log(f"[iommu] translation via VT-d engine '{iommu_engine}' is not enabled")
        else:
            self.logger.log_error("Cannot access VT-d registers")

    def dump_IOMMU_status(self, iommu_engine: str) -> None:
        vtd = IOMMU_ENGINES[iommu_engine]
        self.logger.log('==================================================================')
        self.logger.log(f'[iommu] {iommu_engine} IOMMU Engine Status:')
        self.logger.log('==================================================================')
        if self.cs.register.read(vtd) == 0:
            self.logger.log(f'[iommu] {vtd} value is zero')
            return None
        gsts_reg = self.cs.register.read(f'{vtd}_GSTS')
        self.cs.register.print(f'{vtd}_GSTS', gsts_reg)
        fsts_reg = self.cs.register.read(f'{vtd}_FSTS')
        self.cs.register.print(f'{vtd}_FSTS', fsts_reg)
        frcdl_reg = self.cs.register.read(f'{vtd}_FRCDL')
        self.cs.register.print(f'{vtd}_FRCDL', frcdl_reg)
        frcdh_reg = self.cs.register.read(f'{vtd}_FRCDH')
        self.cs.register.print(f'{vtd}_FRCDH', frcdh_reg)
        ics_reg = self.cs.register.read(f'{vtd}_ICS')
        self.cs.register.print(f'{vtd}_ICS', ics_reg)
        return None



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
This module will attempt to modify the S3 Boot Script on the platform. Doing this could cause the platform to malfunction. Use with care!

Usage:
    Replacing existing opcode::

        chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,<reg_opcode>,<address>,<value>
            <reg_opcode> = pci_wr|mmio_wr|io_wr|pci_rw|mmio_rw|io_rw

        chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,mem[,<address>,<value>]

        chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,dispatch``

        chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,dispatch_ep``


    Adding new opcode::

        chipsec_main.py -m tools.uefi.s3script_modify -a add_op,<reg_opcode>,<address>,<value>,<width>
            <reg_opcode> = pci_wr|mmio_wr|io_wr

        chipsec_main.py -m tools.uefi.s3script_modify -a add_op,dispatch[,<entrypoint>]

Examples:

>>> chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,<reg_opcode>,<address>,<value>
>>>   <reg_opcode> = pci_wr|mmio_wr|io_wr|pci_rw|mmio_rw|io_rw

The option will look for a script opcode that writes to PCI config, MMIO or I/O registers and modify the opcode to write the given value to the register with the given address.

After executing this, if the system is vulnerable to boot script modification, the hardware configuration will have changed according to given <reg_opcode>.

>>> chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,mem

The option will look for a script opcode that writes to memory and modify the opcode to write the given value to the given address.

By default this test will allocate memory and write write ``0xB007B007`` that location.

After executing this, if the system is vulnerable to boot script modification, you should find the given value in the allocated memory location.

>>> chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,dispatch

The option will look for a dispatch opcode in the script and modify the opcode to point to a different entry point. The new entry point will contain a HLT instruction.

After executing this, if the system is vulnerable to boot script modification, the system should hang on resume from S3.

>>> chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,dispatch_ep

The option will look for a dispatch opcode in the script and will modify memory at the entry point for that opcode. The modified instructions will contain a HLT instruction.

After executing this, if the system is vulnerable to dispatch opcode entry point modification, the system should hang on resume from S3.

>>> chipsec_main.py -m tools.uefi.s3script_modify -a add_op,<reg_opcode>,<address>,<value>,<width>
>>>   <reg_opcode> = pci_wr|mmio_wr|io_wr

The option will add a new opcode which writes to PCI config, MMIO or I/O registers with specified values.

>>> chipsec_main.py -m tools.uefi.s3script_modify -a add_op,dispatch

The option will add a new DISPATCH opcode to the script with entry point to either existing or newly allocated memory.

"""

examples_str = """  Examples:
    Replacing existing opcode:
    chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,<reg_opcode>,<address>,<value>
       <reg_opcode> = pci_wr|mmio_wr|io_wr|pci_rw|mmio_rw|io_rw
    chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,mem[,<address>,<value>]
    chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,dispatch
    chipsec_main.py -m tools.uefi.s3script_modify -a replace_op,dispatch_ep

    Adding new opcode:
    chipsec_main.py -m tools.uefi.s3script_modify -a add_op,<reg_opcode>,<address>,<value>,<width>
       <reg_opcode> = pci_wr|mmio_wr|io_wr
    chipsec_main.py -m tools.uefi.s3script_modify -a add_op,dispatch[,<entrypoint>]

"""

import struct

from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.library.logger import print_buffer_bytes
from chipsec.hal.uefi import UEFI
from chipsec.hal.uefi_common import S3BootScriptOpcode, script_width_values, script_width_formats, op_io_pci_mem, op_dispatch
from chipsec.hal.uefi_platform import encode_s3bootscript_entry, id_s3bootscript_type, create_s3bootscript_entry_buffer

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'UEFI', 'S3SCRIPT_MODIFY']

########################################################################################################
#
# Main module functionality
#
########################################################################################################

cmd2opcode = {
    'pci_wr': S3BootScriptOpcode.EFI_BOOT_SCRIPT_PCI_CONFIG_WRITE_OPCODE,
    'mmio_wr': S3BootScriptOpcode.EFI_BOOT_SCRIPT_MEM_WRITE_OPCODE,
    'io_wr': S3BootScriptOpcode.EFI_BOOT_SCRIPT_IO_WRITE_OPCODE,
    'pci_rw': S3BootScriptOpcode.EFI_BOOT_SCRIPT_PCI_CONFIG_READ_WRITE_OPCODE,
    'mmio_rw': S3BootScriptOpcode.EFI_BOOT_SCRIPT_MEM_READ_WRITE_OPCODE,
    'io_rw': S3BootScriptOpcode.EFI_BOOT_SCRIPT_IO_READ_WRITE_OPCODE
}

dispatch_opcode = S3BootScriptOpcode.EFI_BOOT_SCRIPT_DISPATCH_OPCODE
terminate_opcode = S3BootScriptOpcode.EFI_BOOT_SCRIPT_TERMINATE_OPCODE

write_opcodes = [
    S3BootScriptOpcode.EFI_BOOT_SCRIPT_MEM_WRITE_OPCODE,
    S3BootScriptOpcode.EFI_BOOT_SCRIPT_PCI_CONFIG_WRITE_OPCODE,
    S3BootScriptOpcode.EFI_BOOT_SCRIPT_IO_WRITE_OPCODE
]


class s3script_modify(BaseModule):

    DISPATCH_ENTRYPOINT_INSTR = '\x90\x90\xF4\xF4'

    def __init__(self):
        BaseModule.__init__(self)
        self.result.url = 'https://chipsec.github.io/modules/chipsec.modules.tools.uefi.s3script_modify.html'
        self.logger.HAL = True
        self._uefi = UEFI(self.cs)
        self.bootscript_PAs = None
        self.parsed_scripts = None

    def get_bootscript(self):
        if (self.bootscript_PAs is None) or (self.parsed_scripts is None):
            (self.bootscript_PAs, self.parsed_scripts) = self._uefi.get_s3_bootscript(False)
        return (self.bootscript_PAs, self.parsed_scripts)

    def is_supported(self):
        supported = self.cs.helper.EFI_supported()
        if not supported:
            self.logger.log("OS does not support UEFI Runtime API.  Skipping module.")
        else:
            _, ps = self.get_bootscript()
            if not ps:
                self.logger.log("Unable to locate boot script.  Skipping module.")
                supported = False
        return supported

    def modify_s3_reg(self, opcode, address, new_value):
        (bootscript_PAs, parsed_scripts) = self.get_bootscript()
        if parsed_scripts is None:
            self.logger.log_bad("Did not find boot script.")
            return False
        for bootscript_pa in bootscript_PAs:
            if bootscript_pa == 0:
                continue
            self.logger.log(f'[*] Looking for 0x{opcode:X} opcode in the script at 0x{bootscript_pa:016X}..')
            for e in parsed_scripts[bootscript_pa]:
                if (e.decoded_opcode is not None) and \
                   (opcode == e.decoded_opcode.opcode) and \
                   (address == e.decoded_opcode.address):

                    self.logger.log_good(f'Found opcode at offset 0x{e.offset_in_script:04X}')
                    self.logger.log(e)
                    pa = bootscript_pa + e.offset_in_script
                    self.logger.log(f'[*] Modifying S3 boot script entry at address 0x{pa:016X}..')

                    orig_entry_buf = self.cs.mem.read_physical_mem(pa, e.length)
                    self.logger.log("[*] Original entry:")
                    print_buffer_bytes(orig_entry_buf)

                    if opcode in write_opcodes:
                        e.decoded_opcode.values[0] = new_value
                    else:
                        e.decoded_opcode.value = new_value

                    entry_buf = encode_s3bootscript_entry(e)
                    self.cs.mem.write_physical_mem(pa, e.length, entry_buf)

                    new_entry_buf = self.cs.mem.read_physical_mem(pa, e.length)
                    self.logger.log("[*] Modified entry:")
                    print_buffer_bytes(new_entry_buf)
                    return True

        self.logger.log_bad(f'Did not find required 0x{opcode:X} opcode in the script')
        return False

    def modify_s3_dispatch(self):
        ep_size = len(self.DISPATCH_ENTRYPOINT_INSTR)
        (smram_base, _, _) = self.cs.cpu.get_SMRAM()
        (_, new_entrypoint) = self.cs.mem.alloc_physical_mem(ep_size, smram_base)
        self.cs.mem.write_physical_mem(new_entrypoint, ep_size, self.DISPATCH_ENTRYPOINT_INSTR)
        new_ep = self.cs.mem.read_physical_mem(new_entrypoint, ep_size)
        self.logger.log_good(f'Allocated new DISPATCH entry-point at 0x{new_entrypoint:016X} (size = 0x{ep_size:X}):')
        print_buffer_bytes(new_ep)

        (bootscript_PAs, parsed_scripts) = self.get_bootscript()
        if parsed_scripts is None:
            self.logger.log_bad("Did not find boot script.")
            return False
        for bootscript_pa in bootscript_PAs:
            if bootscript_pa == 0:
                continue
            self.logger.log(f'[*] Searching the script at 0x{bootscript_pa:016X} for DISPATCH opcodes..')
            for e in parsed_scripts[bootscript_pa]:
                if (e.decoded_opcode is not None) and (dispatch_opcode == e.decoded_opcode.opcode):

                    self.logger.log_good(f'Found DISPATCH opcode at offset 0x{e.offset_in_script:04X}')
                    self.logger.log(e)
                    pa = bootscript_pa + e.offset_in_script
                    self.logger.log(f'[*] Modifying S3 boot script entry at address 0x{pa:016X}..')

                    orig_entry_buf = self.cs.mem.read_physical_mem(pa, e.length)
                    self.logger.log("[*] Original entry:")
                    print_buffer_bytes(orig_entry_buf)

                    e.decoded_opcode.entrypoint = new_entrypoint
                    entry_buf = encode_s3bootscript_entry(e)
                    self.cs.mem.write_physical_mem(pa, e.length, entry_buf)

                    new_entry_buf = self.cs.mem.read_physical_mem(pa, e.length)
                    self.logger.log("[*] Modified entry:")
                    print_buffer_bytes(new_entry_buf)
                    self.logger.log('After sleep/resume, the system should hang')
                    return True

        self.logger.log_bad("Did not find any suitable DISPATCH opcodes")
        return False

    def modify_s3_dispatch_ep(self):
        ep_pa = None
        (bootscript_PAs, parsed_scripts) = self.get_bootscript()
        if parsed_scripts is None:
            self.logger.log_bad("Did not find boot script.")
            return False
        for script_pa in bootscript_PAs:
            if script_pa == 0:
                continue
            self.logger.log(f'[*] Looking for DISPATCH opcode in the script at 0x{script_pa:016X}..')
            for e in parsed_scripts[script_pa]:
                if (e.decoded_opcode is not None) and (dispatch_opcode == e.decoded_opcode.opcode):
                    ep_pa = e.decoded_opcode.entrypoint
                    self.logger.log_good(f'Found DISPATCH opcode at offset 0x{e.offset_in_script:04X} with entry-point 0x{ep_pa:016X}')
                    self.logger.log(e)
                    break
            if ep_pa is not None:
                break

        if ep_pa is None:
            self.logger.log_bad("Didn't find any DISPATCH opcodes")
            return False

        ep_size = len(self.DISPATCH_ENTRYPOINT_INSTR)
        self.cs.mem.write_physical_mem(ep_pa, ep_size, self.DISPATCH_ENTRYPOINT_INSTR)
        new_ep = self.cs.mem.read_physical_mem(ep_pa, ep_size)
        self.logger.log(f'[*] New DISPATCH entry-point at 0x{ep_pa:016X} (size = 0x{ep_size:X}):')
        print_buffer_bytes(new_ep)
        return True

    def modify_s3_mem(self, address, new_value):
        if address is None:
            (smram_base, _, _) = self.cs.cpu.get_SMRAM()
            (_, address) = self.cs.mem.alloc_physical_mem(0x1000, smram_base)
            self.logger.log(f'[*] Allocated memory at 0x{address:016X} as a target of MEM_WRITE opcode')

        val = self.cs.mem.read_physical_mem_dword(address)
        self.logger.log(f'[*] Original value at 0x{address:016X}: 0x{val:08X}')

        (bootscript_PAs, parsed_scripts) = self.get_bootscript()
        if parsed_scripts is None:
            self.logger.log_bad("Did not find boot script.")
            return False
        for bootscript_pa in bootscript_PAs:
            if bootscript_pa == 0:
                continue
            self.logger.log(f'[*] Looking for MEM_WRITE opcode in the script at 0x{bootscript_pa:016X}..')
            for e in parsed_scripts[bootscript_pa]:
                if (e.decoded_opcode is not None) and (cmd2opcode['mmio_wr'] == e.decoded_opcode.opcode):

                    self.logger.log_good(f'Found opcode at offset 0x{e.offset_in_script:X}')
                    self.logger.log(e)
                    pa = bootscript_pa + e.offset_in_script
                    self.logger.log(f'[*] Modifying S3 boot script entry at address 0x{pa:016X}..')

                    orig_entry_buf = self.cs.mem.read_physical_mem(pa, e.length)
                    self.logger.log("[*] Original entry:")
                    print_buffer_bytes(orig_entry_buf)

                    e.decoded_opcode.address = address
                    e.decoded_opcode.values[0] = new_value
                    entry_buf = encode_s3bootscript_entry(e)
                    self.cs.mem.write_physical_mem(pa, e.length, entry_buf)

                    new_entry_buf = self.cs.mem.read_physical_mem(pa, e.length)
                    self.logger.log("[*] Modified entry:")
                    print_buffer_bytes(new_entry_buf)
                    self.logger.log(f'After sleep/resume, read address 0x{address:08X} and look for value 0x{new_value:08X}')
                    return True

        self.logger.log_bad(f'Did not find required 0x{cmd2opcode["mmio_wr"]:X} opcode in the script')
        return False

    def modify_s3_add(self, new_opcode):
        e_index = None
        (bootscript_PAs, parsed_scripts) = self.get_bootscript()
        if parsed_scripts is None:
            self.logger.log_bad("Did not find boot script.")
            return False

        for bootscript_pa in bootscript_PAs:
            if bootscript_pa == 0:
                continue
            script_buffer = self.cs.mem.read_physical_mem(bootscript_pa, 4)
            script_type, _ = id_s3bootscript_type(script_buffer, False)
            self.logger.log(f'[*] S3 boot script type: 0x{script_type:0X}')

            self.logger.log(f'[*] Looking for TERMINATE opcode in the script at 0x{bootscript_pa:016X}..')
            for e in parsed_scripts[bootscript_pa]:
                if (e.index is not None) and (e.index != -1):
                    e_index = e.index + 1

                if (e.decoded_opcode is not None) and (terminate_opcode == e.decoded_opcode.opcode):
                    self.logger.log_good(f'Found TERMINATE opcode at offset 0x{e.offset_in_script:X}')
                    self.logger.log(e)
                    pa = bootscript_pa + e.offset_in_script
                    orig_entry_buf = self.cs.mem.read_physical_mem(pa, e.length)

                    self.logger.log("[*] New S3 boot script opcode:")
                    self.logger.log(new_opcode)
                    self.logger.log(f'[*] Adding new opcode entry at address 0x{pa:016X}..')
                    new_entry = create_s3bootscript_entry_buffer(script_type, new_opcode, e_index)
                    print_buffer_bytes(new_entry)

                    self.cs.mem.write_physical_mem(pa, len(new_entry), new_entry)
                    last_entry_pa = pa + len(new_entry)
                    self.logger.log(f'[*] Moving TERMINATE opcode to the last entry at 0x{last_entry_pa:016X}..')
                    self.cs.mem.write_physical_mem(last_entry_pa, len(orig_entry_buf), orig_entry_buf)
                    return True

        self.logger.log_bad("Did not find TERMINATE opcode")
        return False

    def run(self, module_argv):
        self.logger.start_test('S3 Resume Boot-Script Testing')
        sts = False

        op = module_argv[0].lower() if len(module_argv) > 0 else 'add_op'
        if op == 'replace_op':
            scmd = module_argv[1].lower() if len(module_argv) > 1 else 'dispatch_ep'
            if scmd in cmd2opcode:
                if len(module_argv) < 4:
                    self.logger.log_error(f'Expected module options: -a replace_op,{scmd},<reg_address>,<value>')
                    self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
                    return self.result.getReturnCode(ModuleResult.ERROR)
                reg_address = int(module_argv[2], 16)
                value = int(module_argv[3], 16)
                sts = self.modify_s3_reg(cmd2opcode[scmd], reg_address, value)
                if sts:
                    self.logger.log(f'[*] After sleep/resume, check the value of register 0x{reg_address:X} is 0x{value:X}')
            elif 'dispatch' == scmd:
                sts = self.modify_s3_dispatch()
            elif 'dispatch_ep' == scmd:
                sts = self.modify_s3_dispatch_ep()
            elif 'mem' == scmd:
                new_value = int(module_argv[2], 16) if len(module_argv) >= 3 else 0xB007B007
                address = int(module_argv[3], 16) if len(module_argv) == 4 else None
                sts = self.modify_s3_mem(address, new_value)
            else:
                self.logger.log_error(f'Unrecognized module command-line argument: {scmd}')
                self.logger.log(examples_str)
                self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
                return self.result.getReturnCode(ModuleResult.ERROR)
        elif op == 'add_op':
            scmd = module_argv[1].lower() if len(module_argv) > 1 else 'dispatch'
            new_opcode = None
            if scmd in cmd2opcode:
                if len(module_argv) < 5:
                    self.logger.log_error(f'Expected module options: -a add_op,{scmd},<reg_address>,<value>,<width>')
                    self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
                    return self.result.getReturnCode(ModuleResult.ERROR)
                address = int(module_argv[2], 16)
                value = int(module_argv[3], 16)
                width = int(module_argv[4], 16)
                width_val = script_width_values[width]
                value_buff = struct.pack(f'<{script_width_formats[width_val]}', value)

                if cmd2opcode[scmd] in write_opcodes:
                    new_opcode = op_io_pci_mem(cmd2opcode[scmd], None, width_val, address, 0, 1, value_buff, None, None)
                else:
                    self.logger.log_error(f'Unsupported opcode: {scmd}')
                    self.logger.log(examples_str)
                    self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
                    return self.result.getReturnCode(ModuleResult.ERROR)
            elif 'dispatch' == scmd:
                if len(module_argv) < 3:
                    (smram_base, _, _) = self.cs.cpu.get_SMRAM()
                    (_, entrypoint) = self.cs.mem.alloc_physical_mem(0x1000, smram_base)
                    self.cs.mem.write_physical_mem(entrypoint, len(self.DISPATCH_ENTRYPOINT_INSTR), self.DISPATCH_ENTRYPOINT_INSTR)
                else:
                    entrypoint = int(module_argv[2], 16)
                new_opcode = op_dispatch(dispatch_opcode, None, entrypoint)
            else:
                self.logger.log_error(f'Unrecognized opcode: {scmd}')
                self.logger.log(examples_str)
                self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
                return self.result.getReturnCode(ModuleResult.ERROR)

            sts = self.modify_s3_add(new_opcode)
        else:
            self.logger.log_error(f'Unrecognized module command-line argument: {op}')
            self.logger.log(examples_str)
            self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
            return self.result.getReturnCode(ModuleResult.ERROR)

        self.result.setStatusBit(self.result.status.VERIFY)

        if sts:
            self.logger.log_passed('The script has been modified. Go to sleep..')
            self.res = ModuleResult.PASSED
        else:
            self.res = ModuleResult.FAILED

        return self.result.getReturnCode(self.res)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Hyper-V specific hypercall functionality
"""

from random import *
from struct import *
from chipsec.modules.tools.vmm.hv.define import *
from chipsec.modules.tools.vmm.common import *
from chipsec.library.logger import *
from chipsec.library.file import *
from chipsec.module_common import *
from chipsec.hal.vmm import *


class HyperVHypercall(BaseModuleHwAccess):
    def __init__(self):
        BaseModuleHwAccess.__init__(self)
        self.hv = VMM(self.cs)
        self.hv.init()
        self.hypervisor_present = False
        self.hv_partitionid = {}
        self.hv_connectionid = {}
        self.hv_hypercalls = {}
        self.param_matrix_status = {}

    ##
    # print_hypervisor_info
    ##
    def print_hypervisor_info(self):
        gprs = self.cpuid_info(0x00000001, 0x0, 'Feature Information')
        self.hypervisor_present = ((gprs[2] >> 31) & 0x1) == 0x1
        self.msg(f'ECX(31) - Hypervisor Present                  :  {self.hypervisor_present:x}')

        if self.hypervisor_present:

            gprs = self.cpuid_info(0x40000000, 0x0, 'Hypervisor CPUID leaf range and vendor ID signature')
            (max_input_value, id_signature_ebx, id_signature_ecx, id_signature_edx) = gprs
            id_signature = pack('<3L', id_signature_ebx, id_signature_ecx, id_signature_edx)
            self.msg(f'The maximum input value for hypervisor CPUID  :  {max_input_value:08X}')
            self.msg(f'Hypervisor Vendor ID Signature                :  {id_signature}')

            gprs = self.cpuid_info(0x40000001, 0x0, 'Hypervisor vendor-neutral interface identification')
            (interface_signature, rsvd_ebx, rsvd_ecx, rsvd_edx) = gprs
            interface_signature = pack('<1L', interface_signature)
            self.msg(f'Hypervisor Interface Signature                :  {interface_signature}')

            if interface_signature == 'Hv#1':
                self.msg('INFO: HV hypervisor CPUID interface detected!')
                if (max_input_value < 0x40000005) or (max_input_value > 0x400000FF):
                    self.msg('')
                    self.msg('*** WARNING ***: Invalid CPUID.0x40000000.0x0.EAX value\n\n')

                for cpuid_eax in range(0x40000002, max_input_value):
                    self.print_hypervisor_cpuid(cpuid_eax)

        return

    ##
    # print_hypervisor_cpuid
    ##
    def print_hypervisor_cpuid(self, cpuid_eax, cpuid_ecx=0x0):
        if cpuid_eax == 0x40000002:
            (eax, ebx, ecx, edx) = self.cpuid_info(cpuid_eax, cpuid_ecx, 'Hypervisor system identity')
            major_version = 0xFFFF & (ebx >> 16)
            minor_version = 0xFFFF & ebx
            service_branch = 0xFF & (edx >> 24)
            service_number = 0xFFFFFF & edx
            self.msg(f'   EAX        - Build Number    :  {eax:08X}')
            self.msg(f'   EBX(31-16) - Major Version   :  {major_version:04X}')
            self.msg(f'   EBX(15-0)  - Minor Version   :  {minor_version:04X}')
            self.msg(f'   ECX        - Service Pack    :  {ecx:08X}')
            self.msg(f'   EDX(31-24) - Service Branch  :  {service_branch:02X}')
            self.msg(f'   EDX(23-0)  - Service Number  :  {service_number:06x}')
        elif cpuid_eax == 0x40000003:
            (eax, ebx, ecx, edx) = self.cpuid_info(cpuid_eax, cpuid_ecx, 'Feature identification')
            self.msg(f'   EAX - features available to the partition                :  {eax:08X}')
            self.info_bitwise(eax, cpuid_desc[cpuid_eax]['EAX'])
            self.msg(f'   EBX - flags specified at partition creation              :  {ebx:08X}')
            self.info_bitwise(ebx, cpuid_desc[cpuid_eax]['EBX'])
            self.msg(f'   ECX - power management related information               :  {ecx:08X}')
            self.info_bitwise(ecx, cpuid_desc[cpuid_eax]['ECX'])
            self.msg(f'   EDX - misc. features available to the partition          :  {edx:08X}')
            self.info_bitwise(edx, cpuid_desc[cpuid_eax]['EDX'])
        elif cpuid_eax == 0x40000004:
            (eax, ebx, ecx, edx) = self.cpuid_info(cpuid_eax, cpuid_ecx, 'Implementation recommendations')
            self.msg(f'   EAX(9-0) - recommendations for optimal performance       :  {eax:08X}')
            self.msg(f'   EBX      - recommended number of attempts                :  {ebx:08X}')
        elif cpuid_eax == 0x40000005:
            (eax, ebx, ecx, edx) = self.cpuid_info(cpuid_eax, cpuid_ecx, 'Implementation limits')
            self.msg(f'   EAX - The maximum number of virtual processors supported :  {eax:08X}')
            self.msg(f'   EBX - The maximum number of logical processors supported :  {ebx:08X}')
            self.msg(f'   ECX - The maximum number of physical interrupt vectors   :  {ecx:08X}')
        elif cpuid_eax == 0x40000006:
            (eax, ebx, ecx, edx) = self.cpuid_info(cpuid_eax, cpuid_ecx, 'Implementation hardware features')
            self.msg(f'   EAX - Intel-specific features  :  {eax:08X}')
            self.msg(f'   EDX - AMD-specific features    :  {edx:08X}')
        else:
            (eax, ebx, ecx, edx) = self.cpuid_info(cpuid_eax, cpuid_ecx, '')
        return

    ##
    # print_synthetic_msrs
    ##
    def print_synthetic_msrs(self):
        self.msg('')
        self.msg('*** Hypervisor Synthetic MSRs ***')

        for addr in sorted(msrs.keys()):
            name = get_msr_name(addr)
            try:
                readValue = self.rdmsr(addr)
                result = f'0x{readValue[0]:08X}_{readValue[1]:08X}'
            except Exception as e:
                result = str(e)
            self.msg(f'RDMSR [{name:40} = 0x{addr:08X}] :  {result}')
        return

    ##
    # scan_hypercalls
    ##
    def scan_hypercalls(self, code_list):
        for call_code in code_list:
            data = self.get_initial_data(GOOD_PARAMS_STATUSES, call_code, 112)
            self.dbg(f'PROBING HYPERCALL: 0x{call_code:04X}')
            for buffer in data:
                try:
                    self.dbg('- FAST HYPERCALL')
                    hciv = hv_hciv(0, 0, call_code, 1)
                    result = self.hv.hypercall64_extended_fast(hciv, buffer) & 0xFFFF
                    hv_rep = 0
                    hv_fast = 1
                    if result == HV_STATUS_INVALID_HYPERCALL_INPUT:
                        self.dbg('- JUST HYPERCALL')
                        hciv = hv_hciv(0, 0, call_code, 0)
                        result = self.hv.hypercall64_memory_based(hciv, buffer) & 0xFFFF
                        hv_fast = 0
                        if result == HV_STATUS_INVALID_HYPERCALL_INPUT:
                            self.dbg('- FAST REP HYPERCALL')
                            hciv = hv_hciv(0, 1, call_code, 1)
                            result = self.hv.hypercall64_extended_fast(hciv, buffer) & 0xFFFF
                            hv_rep = 1
                            if result == HV_STATUS_INVALID_HYPERCALL_INPUT:
                                self.dbg('- REP HYPERCALL')
                                hciv = hv_hciv(0, 1, call_code, 0)
                                result = self.hv.hypercall64_memory_based(hciv, buffer) & 0xFFFF
                                hv_fast = 0
                            else:
                                hv_fast = 1
                    if result != HV_STATUS_INVALID_HYPERCALL_CODE:
                        self.hv_hypercalls[call_code] = [hv_rep, hv_fast, result]
                except Exception as e:
                    self.msg(f'Exception on hypercall (0x{call_code:08X}): {str(e)}')
        return

    ##
    # scan_partitionid
    ##
    def scan_partitionid(self, id_list):
        invalid_partition = 0
        for i in id_list:
            hciv = hv_hciv(0, 0, 0x0041)
            buffer = pack('<Q', i)
            result = self.hv.hypercall64_memory_based(hciv, buffer) & 0xFFFF
            if result == HV_STATUS_ACCESS_DENIED:
                self.hv_partitionid[i] = 1
            if result == HV_STATUS_INVALID_PARTITION_ID:
                invalid_partition = 1
        if invalid_partition == 0:
            self.hv_partitionid = {}
        return

    ##
    # scan_connectionid
    ##
    def scan_connectionid(self, id_list):
        for i in id_list:
            hciv = hv_hciv(0, 0, HV_POST_MESSAGE)
            buffer = pack('<LLLLQ', i, 0x0, 0x1, 8, 0x0)
            result = self.hv.hypercall64_memory_based(hciv, buffer) & 0xFFFF
            if result != HV_STATUS_INVALID_CONNECTION_ID:
                if result == HV_STATUS_SUCCESS:
                    self.hv_connectionid[i] = 0x1
                else:
                    hciv = hv_hciv(0, 0, HV_SIGNAL_EVENT)
                    buffer = pack('<LHH', i, 0x0, 0x0)
                    result = self.hv.hypercall64_memory_based(hciv, buffer) & 0xFFFF
                    self.hv_connectionid[i] = 0x2 if result == HV_STATUS_SUCCESS else 0x3
        return

    ##
    # scan_for_success_status
    ##
    def scan_for_success_status(self, i, total_tests):
        statistics = {}
        pattern = ''
        hc = self.hv_hypercalls[i] if i in self.hv_hypercalls else [0, 0, HV_STATUS_INVALID_HYPERCALL_CODE]
        if hc[2] != HV_STATUS_SUCCESS:
            for x in range(total_tests):
                buffer = ''
                buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
                buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
                buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
                buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
                buffer += '\x00' * 32
                hciv = hv_hciv(0, hc[0], i, 0)
                result = self.hv.hypercall64_memory_based(hciv, buffer)
                rep_completed = (result >> 32) & 0x0FFF
                result = result & 0xFFFF
                statistics[result] = 1 if result not in statistics else statistics[result] + 1
                if result == HV_STATUS_SUCCESS:
                    pattern = buffer
                    break
                if result == HV_STATUS_ACCESS_DENIED:
                    pattern = buffer
            self.msg('*************** Status codes statistics: *****************')
            for n in sorted(statistics.keys()):
                status = get_hypercall_status(n, 'Not defined')
                self.msg(f'{status:50}: {statistics[n]:d}')
            self.hex('Input Parameters', pattern[:0x20])
        else:
            self.msg('')
            self.msg('Hypercall status: SUCCESS')
        return

    ##
    # scan_input_parameters
    ##
    def scan_input_parameters(self, i, maxlen):
        matrix = [[0 for x in range(0x101)] for y in range(maxlen)]
        hc = self.hv_hypercalls[i] if i in self.hv_hypercalls else [0, 0, HV_STATUS_INVALID_HYPERCALL_CODE]
        iv = self.get_initial_data(GOOD_PARAMS_STATUSES, i, 32)[0]
        self.msg('Start scanning ...')
        for l in range(maxlen):
            for v in range(0x100):
                s = list(iv)
                s[l] = chr(v)
                buffer = ''.join(s)
                hciv = hv_hciv(0, hc[0], i, 0)
                result = self.hv.hypercall64_memory_based(hciv, buffer) & 0xFFFF
                matrix[l][v] = result
        self.param_matrix_status[i] = matrix
        self.msg('Done!')
        return

    ##
    # print_input_parameters
    ##
    def print_input_parameters(self, i, maxlen, status_list):
        matrix = self.param_matrix_status[i]
        for l in range(maxlen):
            x = 0
            ranges = []
            for v in range(0x100):
                if (matrix[l][v] not in status_list) and (matrix[l][v + 1] in status_list):
                    x = v + 1
                if (matrix[l][v] in status_list) and (matrix[l][v + 1] not in status_list):
                    if (x == v):
                        ranges.append(f'{x:02X}')
                    else:
                        ranges.append(f'{x:02X}-{v:02X}')
            if (ranges != ['00-FF']) and (ranges != []):
                self.msg(f'  Byte {l:02d} = [{", ".join(ranges)}]')
        return

    ##
    # input_parameters_fuzzing
    ##
    def input_parameters_fuzzing(self, i, maxlen, status_list, total_tests):
        matrix = self.param_matrix_status[i]
        buffer = self.get_initial_data(GOOD_PARAMS_STATUSES, i, 32)[0]
        self.msg('Start input parameters fuzzing ...')
        for x in range(total_tests):
            if x % 10000000 == 10000000 - 1:
                self.msg(f'{100.0 * x / total_tests:4.0f}% DONE')
            l = randint(0, maxlen - 1)
            v = randint(0, 0x100 - 1)
            if matrix[l][v] == 1:
                s = list(buffer)
                s[l] = chr(v)
                buffer = ''.join(s)
                s[randint(0, maxlen - 1)] = chr(randint(0, 0xFF))
                s[randint(0, maxlen - 1)] = chr(randint(0, 0xFF))
                if self.hv_hypercalls[i][0] in status_list:
                    hciv = hv_hciv(0, 1, i, 0)
                else:
                    hciv = hv_hciv(0, 0, i, 0)
                result = self.hv.hypercall64_memory_based(hciv, ''.join(s)) & 0xFFFF
        self.msg('DONE!')
        return

    ##
    # print_hypercall_status
    ##
    def print_hypercall_status(self):
        self.msg('')
        self.msg('*** Hypervisor Hypercall Status Codes ***')
        status_list = [HV_STATUS_INVALID_HYPERCALL_CODE]
        for i in sorted(self.hv_hypercalls.keys()):
            hc = self.hv_hypercalls[i]
            status = get_hypercall_status(hc[2])
            hcname = get_hypercall_name(i)
            if status not in status_list:
                self.msg(f'HYPERV_HYPERCALL REP:{hc[0]:d} FAST:{hc[1]:d} {i:04X}  {hc[2]:02X}  {status:40} \'{hcname}\'')
        return

    ##
    # print_partitionid
    ##
    def print_partitionid(self):
        self.msg('')
        self.msg('*** Hypervisor Partition IDs ***')
        if len(self.hv_partitionid) == 0:
            self.msg('  was not able to determine Partition IDs')
        else:
            for i in sorted(self.hv_partitionid.keys()):
                self.msg(f'{i:08X}')
        return

    ##
    # print_partition_properties
    ##
    def print_partition_properties(self):
        self.msg('')
        self.msg('*** Partition properties ***')
        for partid in sorted(self.hv_partitionid.keys()):
            for n in range(0x10):
                for m in range(0x10):
                    hciv = hv_hciv(0, 0, HV_GET_PARTITION_PROPERTY)
                    prop = (n << 16) + m
                    buffer = pack('<QLL', partid, prop, 0)
                    result = self.hv.hypercall64_memory_based(hciv, buffer, 8)
                    if result == HV_STATUS_SUCCESS:
                        self.msg(f'  Partition: {partid:08X}  Property: {prop:08X}  Value: {unpack("<Q", self.hv.output)[0]:016x}')
        return

    ##
    # set_partition_property
    ##
    def set_partition_property(self, part, prop, value):
        hciv = hv_hciv(0, 0, HV_SET_PARTITION_PROPERTY)
        buffer = pack('<QLLQ', part, prop, 0, value)
        result = self.hv.hypercall64_memory_based(hciv, buffer)
        status = get_hypercall_status(result, f'0x{result:08X}')
        self.msg(f'>>> Setting partition property:  Partition: {part:08X}  Property: {prop:08X}  Value: {value:016x}  Status: {status}')
        return

    ##
    # print_connectionid
    ##
    def print_connectionid(self, status_list):
        self.msg('')
        self.msg('*** Hypervisor Connection IDs ***')
        for i in sorted(self.hv_connectionid.keys()):
            connid = self.hv_connectionid[i]
            self.msg(f'{i:08X}  {connid:02X}  {hv_porttype[connid]}')
        return

    ##
    # custom_fuzzing
    ##
    def custom_fuzzing(self, call_code, total_tests):
        statistics = {}
        buffer = ''.join([chr(randint(0, 255)) for i in range(0, 112)])
        hcname = get_hypercall_name(call_code)

        if hcname == 'HvConnectPort':
            return
            self.msg(f'Hypercall: {hcname} ')
            hciv = hv_hciv(0, 0, call_code)
            for i in range(0x0, 0xFFFFF):
                buffer = pack('<5Q', i & 0xF, (i >> 4) & 0xF, (i >> 8) & 0xF, (i >> 12) & 0xF, (i >> 16) & 0xF)
                result = self.hv.hypercall64_memory_based(hciv, buffer)
                statistics[result] = 1 if result not in statistics else statistics[result] + 1

        elif hcname == 'HvPostMessage':
            self.msg(f'Hypercall: {hcname} ')
            hciv = hv_hciv(0, 0, call_code)
            for connid in sorted(self.hv_connectionid.keys()):
                if self.hv_connectionid[connid] == HV_PORT_TYPE_MESSAGE:
                    self.msg(f'Connection ID: : {connid:08X}')
                    for i in range(0x100, 0x1000):
                        messagetype = 0x7FFFFFFF & getrandbits(32)
                        payloadsize = randint(0, 240)
                        message0 = getrandbits(8)
                        buffer = pack('<4LQ', connid, 0, messagetype, payloadsize, message0)
                        result = hv.hypercall64_memory_based(hciv, buffer)
                        statistics[result] = 1 if result not in statistics else statistics[result] + 1
                        self.dbg(f'HvPostMessage: {get_hypercall_status(result)} {buffer.hex()}')

        elif hcname == 'HvSignalEvent':
            self.msg(f'Hypercall: {hcname} ')
            hciv = hv_hciv(0, 0, call_code)
            for connid in sorted(self.hv_connectionid.keys()):
                if self.hv_connectionid[connid] == HV_PORT_TYPE_EVENT:
                    self.msg(f'Connection ID: {connid:08X}')
                    for i in range(min(total_tests, 0xFFFF)):
                        buffer = pack('<LHH', connid, 0, i)
                        result = self.hv.hypercall64_memory_based(hciv, buffer)
                        statistics[result] = 1 if result not in statistics else statistics[result] + 1

        if len(statistics) > 0:
            self.msg('*************** Status codes statistics: *****************')
            for i in sorted(statistics.keys()):
                self.msg(f'{get_hypercall_status(i):50}: {statistics[i]:d}')

        return



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
The mem command provides direct access to read and write physical memory.

>>> chipsec_util mem <op> <physical_address> <length> [value|buffer_file]
>>> <physical_address> : 64-bit physical address
>>> <op>               : read|readval|write|writeval|allocate|pagedump|search
>>> <length>           : byte|word|dword or length of the buffer from <buffer_file>
>>> <value>            : byte, word or dword value to be written to memory at <physical_address>
>>> <buffer_file>      : file with the contents to be written to memory at <physical_address>

Examples:

>>> chipsec_util mem <op>     <physical_address> <length> [value|file]
>>> chipsec_util mem readval  0xFED40000         dword
>>> chipsec_util mem read     0x41E              0x20     buffer.bin
>>> chipsec_util mem writeval 0xA0000            dword    0x9090CCCC
>>> chipsec_util mem write    0x100000000        0x1000   buffer.bin
>>> chipsec_util mem write    0x100000000        0x10     000102030405060708090A0B0C0D0E0F
>>> chipsec_util mem allocate                    0x1000
>>> chipsec_util mem pagedump 0xFED00000         0x100000
>>> chipsec_util mem search   0xF0000            0x10000  _SM_
"""

import os

from chipsec.command import BaseCommand, toLoad
from chipsec.library.defines import ALIGNED_4KB, BOUNDARY_4KB, bytestostring
from chipsec_util import get_option_width, is_option_valid_width, CMD_OPTS_WIDTH
from chipsec.library.file import read_file, write_file, get_main_dir
from chipsec.library.logger import print_buffer_bytes
from argparse import ArgumentParser

# Physical Memory


class MemCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.Driver

    def parse_arguments(self) -> None:
        parser = ArgumentParser(prog='chipsec_util mem', usage=__doc__)
        subparsers = parser.add_subparsers()

        parser_read = subparsers.add_parser('read')
        parser_read.add_argument('phys_address', type=lambda x: int(x, 16), help='64-bit physical address (hex)')
        parser_read.add_argument('buffer_length', type=lambda x: int(x, 16), default=0x100, nargs='?', help='Length of buffer (hex)')
        parser_read.add_argument('file_name', type=str, default='', nargs='?', help='Buffer file name')
        parser_read.set_defaults(func=self.mem_read)

        parser_readval = subparsers.add_parser('readval')
        parser_readval.add_argument('phys_address', type=lambda x: int(x, 16), help='64-bit physical address (hex)')
        parser_readval.add_argument('length', type=str, nargs='?', default='', help='Length to read (byte|word|dword)')
        parser_readval.set_defaults(func=self.mem_readval)

        parser_write = subparsers.add_parser('write')
        parser_write.add_argument('phys_address', type=lambda x: int(x, 16), help='64-bit physical address (hex)')
        parser_write.add_argument('buffer_length', type=lambda x: int(x, 16), help='Length of buffer (hex)')
        parser_write.add_argument('buffer_data', type=str, help='Buffer data or file name')
        parser_write.set_defaults(func=self.mem_write)

        parser_writeval = subparsers.add_parser('writeval')
        parser_writeval.add_argument('phys_address', type=lambda x: int(x, 16), help='64-bit physical address (hex)')
        parser_writeval.add_argument('length', type=str, help='Length to write (byte|word|dword)')
        parser_writeval.add_argument('write_data', type=lambda x: int(x, 16), help='Data to write')
        parser_writeval.set_defaults(func=self.mem_writeval)

        parser_allocate = subparsers.add_parser('allocate')
        parser_allocate.add_argument('allocate_length', type=lambda x: int(x, 16), help='Length to allocate (hex)')
        parser_allocate.set_defaults(func=self.mem_allocate)

        parser_pagedump = subparsers.add_parser('pagedump')
        parser_pagedump.add_argument('start_address', type=lambda x: int(x, 16), help='64-bit physical address (hex)')
        parser_pagedump.add_argument('length', type=lambda x: int(x, 16), nargs='?', default=BOUNDARY_4KB, help='Length to allocate (hex)')
        parser_pagedump.set_defaults(func=self.mem_pagedump)

        parser_search = subparsers.add_parser('search')
        parser_search.add_argument('phys_address', type=lambda x: int(x, 16), help='64-bit physical address (hex)')
        parser_search.add_argument('length', type=lambda x: int(x, 16), help='Length to search (hex)')
        parser_search.add_argument('value', type=str, help='Value to search for')
        parser_search.set_defaults(func=self.mem_search)
        parser.parse_args(self.argv, namespace=self)

    def dump_region_to_path(self, path: str, pa_start: int, pa_end: int) -> None:
        if pa_start >= pa_end:
            return
        head_len = pa_start & ALIGNED_4KB
        tail_len = pa_end & ALIGNED_4KB
        pa = pa_start - head_len + ALIGNED_4KB + 1
        fname = os.path.join(path, f'm{pa_start:016X}.bin')
        end = pa_end - tail_len
        with open(fname, 'wb') as f:
            # read leading bytes to the next boundary
            if (head_len > 0):
                f.write(self.cs.mem.read_physical_mem(pa_start, ALIGNED_4KB + 1 - head_len))

            for addr in range(pa, end, ALIGNED_4KB + 1):
                f.write(self.cs.mem.read_physical_mem(addr, ALIGNED_4KB + 1))

            # read trailing bytes
            if (tail_len > 0):
                f.write(self.cs.mem.read_physical_mem(end, tail_len))

    def mem_allocate(self) -> None:
        (va, pa) = self.cs.mem.alloc_physical_mem(self.allocate_length)
        self.logger.log(f'[CHIPSEC] Allocated {self.allocate_length:X} bytes of physical memory: VA = 0x{va:016X}, PA = 0x{pa:016X}')

    def mem_search(self) -> None:
        buffer = self.cs.mem.read_physical_mem(self.phys_address, self.length)
        buffer = bytestostring(buffer)
        offset = buffer.find(self.value)

        if (offset != -1):
            self.logger.log(f'[CHIPSEC] Search buffer from memory: PA = 0x{self.phys_address:016X}, len = 0x{self.length:X}, target address= 0x{self.phys_address + offset:X}..')
        else:
            self.logger.log(f'[CHIPSEC] Search buffer from memory: PA = 0x{self.phys_address:016X}, len = 0x{self.length:X}, can not find the target in the searched range..')

    def mem_pagedump(self) -> None:
        end = self.start_address + self.length
        self.dump_region_to_path(get_main_dir(), self.start_address, end)

    def mem_read(self) -> None:
        self.logger.log(f'[CHIPSEC] Reading buffer from memory: PA = 0x{self.phys_address:016X}, len = 0x{self.buffer_length:X}..')
        buffer = self.cs.mem.read_physical_mem(self.phys_address, self.buffer_length)
        if self.file_name:
            write_file(self.file_name, buffer)
            self.logger.log(f'[CHIPSEC] Written 0x{len(buffer):X} bytes to \'{self.file_name}\'')
        else:
            print_buffer_bytes(buffer)

    def mem_readval(self) -> None:
        width = 0x4
        value = 0x0
        if self.length:
            try:
                width = get_option_width(self.length) if is_option_valid_width(self.length) else int(self.length, 16)
            except ValueError:
                self.logger.log_error(f'[CHIPSEC] Bad length given \'{self.length}\'')
                return

        if width not in (0x1, 0x2, 0x4):
            self.logger.log_error(f'Must specify <length> argument in \'mem readval\' as one of {CMD_OPTS_WIDTH}')
            return
        self.logger.log(f'[CHIPSEC] Reading {width:X}-byte value from PA 0x{self.phys_address:016X}..')
        if 0x1 == width:
            value = self.cs.mem.read_physical_mem_byte(self.phys_address)
        elif 0x2 == width:
            value = self.cs.mem.read_physical_mem_word(self.phys_address)
        elif 0x4 == width:
            value = self.cs.mem.read_physical_mem_dword(self.phys_address)
        self.logger.log(f'[CHIPSEC] Value = 0x{value:X}')

    def mem_write(self) -> None:
        if not os.path.exists(self.buffer_data):
            try:
                buffer = bytearray.fromhex(self.buffer_data)
            except ValueError:
                self.logger.log_error(f'Incorrect <value> specified: \'{self.buffer_data}\'')
                return
            self.logger.log(f'[CHIPSEC] Read 0x{len(buffer):X} hex bytes from command-line: \'{self.buffer_data}\'')
        else:
            buffer = read_file(self.buffer_data)
            self.logger.log(f'[CHIPSEC] Read 0x{len(buffer):X} bytes from file \'{self.buffer_data}\'')

        if len(buffer) < self.buffer_length:
            self.logger.log_error(f'Number of bytes read (0x{len(buffer):X}) is less than the specified <length> (0x{self.buffer_length:X})')
            return

        self.logger.log(f'[CHIPSEC] writing buffer to memory: PA = 0x{self.phys_address:016X}, len = 0x{self.buffer_length:X}..')
        self.cs.mem.write_physical_mem(self.phys_address, self.buffer_length, buffer)

    def mem_writeval(self) -> None:
        try:
            width = get_option_width(self.length) if is_option_valid_width(self.length) else int(self.length, 16)
        except ValueError:
            self.logger.log_error(f'Must specify <length> argument in \'mem writeval\' as one of {CMD_OPTS_WIDTH}')
            return

        if width not in (0x1, 0x2, 0x4):
            self.logger.log_error(f'Must specify <length> argument in \'mem writeval\' as one of {CMD_OPTS_WIDTH}')
            return
        self.logger.log(f'[CHIPSEC] Writing {width:X}-byte value 0x{self.write_data:X} to PA 0x{self.phys_address:016X}..')
        if 0x1 == width:
            self.cs.mem.write_physical_mem_byte(self.phys_address, self.write_data)
        elif 0x2 == width:
            self.cs.mem.write_physical_mem_word(self.phys_address, self.write_data)
        elif 0x4 == width:
            self.cs.mem.write_physical_mem_dword(self.phys_address, self.write_data)


commands = {'mem': MemCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2024, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

import os
import fnmatch
from chipsec.library.logger import logger
from chipsec.library.file import get_module_dir
from typing import List


def enumerate_modules() -> List[str]:
    mod_path = get_module_dir()
    tools_path = os.path.join(mod_path)
    files = []
    for dirname, _, mod_fnames in os.walk(os.path.abspath(tools_path)):
        for modx in mod_fnames:
            if fnmatch.fnmatch(modx, '*.py') and not fnmatch.fnmatch(modx, '__init__.py'):
                module_path = os.path.relpath(dirname, mod_path).replace('\\', '.').replace('/', '.')
                files.append(f'{module_path}.{modx[:-3]}')
    return files


def print_modules(module_list: List[str]) -> None:
    logger().log('Enumerating modules...')
    for module in module_list:
        logger().log(f'\t{module}')



#CHIPSEC: Platform Security Assessment Framework
#Copyright (c) 2010-2015, Intel Corporation
# 
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; Version 2.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#Contact information:
#chipsec@intel.com
#



###\defgroup  core Core Components
# Components providing core functionality
